@@ -28,12 +28,14 @@ module Text.Pandoc.Class.PandocMonad
   , getZonedTime
   , readFileFromDirs
   , report
-  , setTrace
+  , runSilently
   , setRequestHeader
   , setNoCheckCertificate
   , getLog
   , setVerbosity
   , getVerbosity
+  , setTrace
+  , getTrace
   , getMediaBag
   , setMediaBag
   , insertMedia
@@ -47,6 +49,9 @@ module Text.Pandoc.Class.PandocMonad
   , setOutputFile
   , setResourcePath
   , getResourcePath
+  , setRequestHeaders
+  , getRequestHeaders
+  , getSourceURL
   , readMetadataFile
   , toTextM
   , fillMediaBag
@@ -164,6 +169,15 @@ setVerbosity verbosity =
 getVerbosity :: PandocMonad m => m Verbosity
 getVerbosity = getsCommonState stVerbosity
 
+-- | Set tracing. This affects the behavior of 'trace'. If tracing
+-- is not enabled, 'trace' does nothing.
+setTrace :: PandocMonad m => Bool -> m ()
+setTrace enabled = modifyCommonState $ \st -> st{ stTrace = enabled }
+
+-- | Get tracing status.
+getTrace :: PandocMonad m => m Bool
+getTrace = getsCommonState stTrace
+
 -- | Get the accumulated log messages (in temporal order).
 getLog :: PandocMonad m => m [LogMessage]
 getLog = reverse <$> getsCommonState stLog
@@ -179,11 +193,22 @@ report msg = do
   when (level <= verbosity) $ logOutput msg
   modifyCommonState $ \st -> st{ stLog = msg : stLog st }
 
--- | Determine whether tracing is enabled.  This affects
--- the behavior of 'trace'.  If tracing is not enabled,
--- 'trace' does nothing.
-setTrace :: PandocMonad m => Bool -> m ()
-setTrace useTracing = modifyCommonState $ \st -> st{stTrace = useTracing}
+-- | Run an action, but suppress the output of any log messages;
+-- instead, all messages reported by @action@ are returned separately
+-- and not added to the main log.
+runSilently :: PandocMonad m => m a -> m (a, [LogMessage])
+runSilently action = do
+  -- get current settings
+  origLog <- getsCommonState stLog
+  origVerbosity <- getVerbosity
+  -- reset log level and set verbosity to the minimum
+  modifyCommonState (\st -> st { stVerbosity = ERROR, stLog = []})
+  result <- action
+  -- get log messages reported while running `action`
+  newLog <- getsCommonState stLog
+  modifyCommonState (\st -> st { stVerbosity = origVerbosity, stLog = origLog})
+
+  return (result, newLog)
 
 -- | Set request header to use in HTTP requests.
 setRequestHeader :: PandocMonad m
@@ -248,6 +273,18 @@ getResourcePath = getsCommonState stResourcePath
 setResourcePath :: PandocMonad m => [FilePath] -> m ()
 setResourcePath ps = modifyCommonState $ \st -> st{stResourcePath = ps}
 
+-- | Retrieve the request headers to add for HTTP requests.
+getRequestHeaders :: PandocMonad m => m [(T.Text, T.Text)]
+getRequestHeaders = getsCommonState stRequestHeaders
+
+-- | Set the request headers to add for HTTP requests.
+setRequestHeaders :: PandocMonad m => [(T.Text, T.Text)] -> m ()
+setRequestHeaders hs = modifyCommonState $ \st -> st{ stRequestHeaders = hs }
+
+-- | Get the absolute UL or directory of first source file.
+getSourceURL :: PandocMonad m => m (Maybe T.Text)
+getSourceURL = getsCommonState stSourceURL
+
 -- | Get the current UTC time. If the @SOURCE_DATE_EPOCH@ environment
 -- variable is set to a unix time (number of seconds since midnight
 -- Jan 01 1970 UTC), it is used instead of the current time, to support
@@ -371,8 +408,7 @@ downloadOrRead s
                           uriPath = "",
                           uriQuery = "",
                           uriFragment = "" }
-         dropFragmentAndQuery = T.takeWhile (\c -> c /= '?' && c /= '#')
-         fp = unEscapeString $ T.unpack $ dropFragmentAndQuery s
+         fp = unEscapeString $ T.unpack s
          mime = getMimeType $ case takeExtension fp of
                      ".gz" -> dropExtension fp
                      ".svgz" -> dropExtension fp ++ ".svg"
