@@ -59,8 +59,6 @@ data Extension =
     | Ext_blank_before_header     -- ^ Require blank line before a header
     | Ext_bracketed_spans         -- ^ Bracketed spans with attributes
     | Ext_citations           -- ^ Pandoc/citeproc citations
-    | Ext_compact_definition_lists  -- ^ Definition lists without space between items,
-                                    --   and disallow laziness
     | Ext_definition_lists    -- ^ Definition lists as in pandoc, mmd, php
     | Ext_east_asian_line_breaks  -- ^ Newlines in paragraphs are ignored between
                                   --   East Asian wide characters. Note: this extension
@@ -122,6 +120,8 @@ data Extension =
     | Ext_shortcut_reference_links -- ^ Shortcut reference links
     | Ext_simple_tables       -- ^ Pandoc-style simple tables
     | Ext_smart               -- ^ "Smart" quotes, apostrophes, ellipses, dashes
+    | Ext_smart_quotes        -- ^ "Smart" quotes
+    | Ext_special_strings     -- ^ Treat certain strings like special characters
     | Ext_sourcepos           -- ^ Include source position attributes
     | Ext_space_in_atx_header -- ^ Require space between # and header text
     | Ext_spaced_reference_links -- ^ Allow space between two parts of ref link
@@ -431,6 +431,7 @@ getDefaultExtensions "commonmark_x"    = extensionsFromList
   ]
 getDefaultExtensions "org"             = extensionsFromList
                                           [Ext_citations,
+                                           Ext_special_strings,
                                            Ext_task_lists,
                                            Ext_auto_identifiers]
 getDefaultExtensions "html"            = extensionsFromList
@@ -511,7 +512,6 @@ getAllExtensions f = universalExtensions <> getAll f
        , Ext_mark
        , Ext_mmd_link_attributes
        , Ext_mmd_header_identifiers
-       , Ext_compact_definition_lists
        , Ext_gutenberg
        , Ext_smart
        , Ext_literate_haskell
@@ -583,6 +583,8 @@ getAllExtensions f = universalExtensions <> getAll f
     extensionsFromList
     [ Ext_citations
     , Ext_smart
+    , Ext_smart_quotes
+    , Ext_special_strings
     , Ext_fancy_lists
     , Ext_task_lists
     ]
