@@ -1,7 +1,7 @@
 {-# LANGUAGE OverloadedStrings #-}
 {- |
    Module      : Text.Pandoc.Highlighting
-   Copyright   : Copyright (C) 2008-2022 John MacFarlane
+   Copyright   : Copyright (C) 2008-2025 John MacFarlane
    License     : GNU GPL, version 2 or above
 
    Maintainer  : John MacFarlane <jgm@berkeley.edu>
@@ -15,12 +15,23 @@ module Text.Pandoc.Highlighting ( highlightingStyles
                                 , languages
                                 , languagesByExtension
                                 , highlight
+                                -- * Formats
+                                -- ** LaTeX
                                 , formatLaTeXInline
                                 , formatLaTeXBlock
                                 , styleToLaTeX
+                                -- ** HTML
                                 , formatHtmlInline
                                 , formatHtmlBlock
+                                , formatHtml4Block
                                 , styleToCss
+                                -- ** ConTeXt
+                                , formatConTeXtInline
+                                , formatConTeXtBlock
+                                , styleToConTeXt
+                                , formatANSI
+                                -- * Styles
+                                , defaultStyle
                                 , pygments
                                 , espresso
                                 , zenburn
@@ -30,6 +41,7 @@ module Text.Pandoc.Highlighting ( highlightingStyles
                                 , breezeDark
                                 , haddock
                                 , Style
+                                , lookupHighlightingStyle
                                 , fromListingsLanguage
                                 , toListingsLanguage
                                 ) where
@@ -39,8 +51,16 @@ import Data.Maybe (fromMaybe)
 import qualified Data.Text as T
 import Skylighting
 import Text.Pandoc.Definition
+import Text.Pandoc.Class (PandocMonad, readFileLazy)
+import Text.Pandoc.Error (PandocError(..))
+import Control.Monad.Except (throwError)
+import System.FilePath (takeExtension)
 import Text.Pandoc.Shared (safeRead)
 
+-- | The default highlighting style used by pandoc (pygments).
+defaultStyle :: Style
+defaultStyle = pygments
+
 highlightingStyles :: [(T.Text, Style)]
 highlightingStyles =
   [("pygments", pygments),
@@ -214,3 +234,20 @@ toListingsLanguage lang = M.lookup (T.toLower lang) langToListingsMap
 -- | Determine skylighting language name from listings language name.
 fromListingsLanguage :: T.Text -> Maybe T.Text
 fromListingsLanguage lang = M.lookup lang listingsToLangMap
+
+-- | Lookup style from a name. If the name is a standard style,
+-- load it; if it ends in ".theme", attempt to load a KDE theme
+-- from the file path specified.
+lookupHighlightingStyle :: PandocMonad m => String -> m Style
+lookupHighlightingStyle s
+  | takeExtension s == ".theme" = -- attempt to load KDE theme
+    do contents <- readFileLazy s
+       case parseTheme contents of
+            Left _    -> throwError $ PandocOptionError $ T.pack $
+                           "Could not read highlighting theme " ++ s
+            Right sty -> return sty
+  | otherwise =
+  case lookup (T.toLower $ T.pack s) highlightingStyles of
+       Just sty -> return sty
+       Nothing  -> throwError $ PandocOptionError $ T.pack $
+                      "Unknown highlight-style " ++ s
