@@ -38,7 +38,7 @@ import qualified Data.ByteString.Char8 as B
 import qualified Data.ByteString.Lazy as BL
 import Data.Binary.Get
 import Data.Bits ((.&.), shiftR, shiftL)
-import Data.Word (bitReverse32)
+import Data.Word (bitReverse32, Word32)
 import Data.Maybe (isJust, fromJust)
 import Data.Char (isDigit)
 import Control.Monad
@@ -57,11 +57,12 @@ import qualified Data.Attoparsec.ByteString as AW
 import qualified Data.Attoparsec.ByteString.Char8 as A
 import qualified Codec.Picture.Metadata as Metadata
 import Codec.Picture (decodeImageWithMetadata)
+import Codec.Compression.Zlib (decompress)
+-- import Debug.Trace
 
 -- quick and dirty functions to get image sizes
--- algorithms borrowed from wwwis.pl
 
-data ImageType = Png | Gif | Jpeg | Svg | Pdf | Eps | Emf | Tiff | Webp
+data ImageType = Png | Gif | Jpeg | Svg | Pdf | Eps | Emf | Tiff | Webp | Avif
                  deriving Show
 data Direction = Width | Height
 instance Show Direction where
@@ -72,6 +73,8 @@ data Dimension = Pixel Integer
                | Centimeter Double
                | Millimeter Double
                | Inch Double
+               | Point Double
+               | Pica Double
                | Percent Double
                | Em Double
                deriving Eq
@@ -81,6 +84,8 @@ instance Show Dimension where
   show (Centimeter a) = T.unpack (showFl a) ++ "cm"
   show (Millimeter a) = T.unpack (showFl a) ++ "mm"
   show (Inch a)       = T.unpack (showFl a) ++ "in"
+  show (Point a)      = T.unpack (showFl a) ++ "pt"
+  show (Pica a)       = T.unpack (showFl a) ++ "pc"
   show (Percent a)    = show a              ++ "%"
   show (Em a)         = T.unpack (showFl a) ++ "em"
 
@@ -113,8 +118,11 @@ imageType img = case B.take 4 img of
                      "\x47\x49\x46\x38" -> return Gif
                      "\x49\x49\x2a\x00" -> return Tiff
                      "\x4D\x4D\x00\x2a" -> return Tiff
-                     "\xff\xd8\xff\xe0" -> return Jpeg  -- JFIF
-                     "\xff\xd8\xff\xe1" -> return Jpeg  -- Exif
+                     "\xff\xd8\xff\xbd" -> return Jpeg  -- JPEG without application segment -- see p.32 in https://www.w3.org/Graphics/JPEG/itu-t81.pdf (and https://gist.github.com/leommoore/f9e57ba2aa4bf197ebc5?permalink_comment_id=3863054#gistcomment-3863054)
+                     _ | B.take 3 img == "\xff\xd8\xff"
+                          && (let byte4 = B.take 1 (B.drop 3 img)
+                              in byte4 >= "\xe0" && byte4 <= "\xef")  -- JPEG with application segment
+                                        -> return Jpeg
                      "%PDF"             -> return Pdf
                      "<svg"             -> return Svg
                      "<?xm"
@@ -131,6 +139,7 @@ imageType img = case B.take 4 img of
                      "RIFF"
                        | B.take 4 (B.drop 8 img) == "WEBP"
                                         -> return Webp
+                     _ | B.take 4 (B.drop 4 img) == "ftyp" -> return Avif
                      _ -> mzero
 
 findSvgTag :: ByteString -> Bool
@@ -148,6 +157,7 @@ imageSize opts img = checkDpi <$>
        Just Pdf  -> mbToEither "could not determine PDF size" $ pdfSize img
        Just Emf  -> mbToEither "could not determine EMF size" $ emfSize img
        Just Webp -> mbToEither "could not determine WebP size" $ webpSize opts img
+       Just Avif -> mbToEither "could not determine AVIF size" $ avifSize opts img
        Nothing   -> Left "could not determine image type"
   where mbToEither msg Nothing  = Left msg
         mbToEither _   (Just x) = Right x
@@ -201,6 +211,8 @@ inInch opts dim =
     (Centimeter a) -> a * 0.3937007874
     (Millimeter a) -> a * 0.03937007874
     (Inch a)       -> a
+    (Point a)      -> (a / 72)
+    (Pica a)       -> (a / 6)
     (Percent _)    -> 0
     (Em a)         -> a * (11/64)
 
@@ -208,11 +220,7 @@ inPixel :: WriterOptions -> Dimension -> Integer
 inPixel opts dim =
   case dim of
     (Pixel a)      -> a
-    (Centimeter a) -> floor $ dpi * a * 0.3937007874 :: Integer
-    (Millimeter a) -> floor $ dpi * a * 0.03937007874 :: Integer
-    (Inch a)       -> floor $ dpi * a :: Integer
-    (Percent _)    -> 0
-    (Em a)         -> floor $ dpi * a * (11/64) :: Integer
+    _              -> floor (dpi * inInch opts dim)
   where
     dpi = fromIntegral $ writerDpi opts
 
@@ -242,6 +250,8 @@ scaleDimension factor dim =
         Centimeter x -> Centimeter (factor * x)
         Millimeter x -> Millimeter (factor * x)
         Inch x       -> Inch (factor * x)
+        Point x      -> Point (factor * x)
+        Pica x       -> Pica (factor * x)
         Percent x    -> Percent (factor * x)
         Em x         -> Em (factor * x)
 
@@ -265,8 +275,8 @@ lengthToDim s = numUnit s >>= uncurry toDim
     toDim a "%"    = Just $ Percent a
     toDim a "px"   = Just $ Pixel (floor a::Integer)
     toDim a ""     = Just $ Pixel (floor a::Integer)
-    toDim a "pt"   = Just $ Inch (a / 72)
-    toDim a "pc"   = Just $ Inch (a / 6)
+    toDim a "pt"   = Just $ Point a
+    toDim a "pc"   = Just $ Pica a
     toDim a "em"   = Just $ Em a
     toDim _ _      = Nothing
 
@@ -294,10 +304,10 @@ pdfSize img =
     Right sz -> Just sz
 
 pPdfSize :: A.Parser ImageSize
-pPdfSize = do
-  A.skipWhile (/='/')
-  A.char8 '/'
-  (do A.string "MediaBox"
+pPdfSize =
+  (A.takeWhile1 (/= '/') *> pPdfSize)
+  <|>
+  (do A.string "/MediaBox"
       A.skipSpace
       A.char8 '['
       A.skipSpace
@@ -314,7 +324,27 @@ pPdfSize = do
             , pxY  = y2 - y1
             , dpiX = 72
             , dpiY = 72 }
-   ) <|> pPdfSize
+  )
+  <|> -- if we encounter a compressed object stream, uncompress it (#10902)
+  (do A.string "/Type"
+      A.skipSpace
+      A.string "/ObjStm"
+      _ <- A.manyTill pLine (A.string "stream" *> pEol)
+      stream <- BL.pack <$> A.manyTill
+                        (AW.satisfy (const True))
+                        (pEol *> A.string "endstream" *> pEol)
+      let contents = BL.toStrict (decompress stream)
+      case A.parseOnly pPdfSize contents of
+        Left _ -> pPdfSize
+        Right is -> pure is)
+  <|>
+  (A.char '/' *> pPdfSize)
+ where
+   iseol '\r' = True
+   iseol '\n' = True
+   iseol _ = False
+   pEol = A.satisfy iseol *> A.skipMany (A.satisfy iseol)
+   pLine = A.takeWhile (not . iseol) <* pEol
 
 getSize :: ByteString -> Either T.Text ImageSize
 getSize img =
@@ -451,3 +481,173 @@ webpSize opts img =
   case AW.parseOnly pWebpSize img of
     Left _   -> Nothing
     Right sz -> Just sz { dpiX = fromIntegral $ writerDpi opts, dpiY = fromIntegral $ writerDpi opts}
+
+avifSize :: WriterOptions -> ByteString -> Maybe ImageSize
+avifSize _opts img =
+  case runGetOrFail (verifyFtyp >> findAvifDimensions) (BL.fromStrict img) of
+    Left (_, _, _err) -> Nothing
+    Right (_, _, (width, height)) ->
+      Just $ ImageSize { pxX = fromIntegral width
+                       , pxY = fromIntegral height
+                       , dpiX = 72
+                       , dpiY = 72 }
+
+---- AVIF parsing:
+
+verifyFtyp :: Get ()
+verifyFtyp = do
+  ftypSize <- getWord32be
+  when (ftypSize < 16) $ fail "Invalid ftyp size"
+
+  ftyp <- getByteString 4
+  unless (ftyp == "ftyp") $ fail "ftyp signature not found"
+
+  brand <- getByteString 4
+  unless (brand == "avif" || brand == "avis") $ fail "Not an AVIF file"
+
+  -- Skip minor version and compatible brands
+  -- (we've read 12 bytes: size+type+brand)
+  let remaining_ftyp = fromIntegral ftypSize - 12
+  when (remaining_ftyp > 0) $ skip remaining_ftyp
+
+findAvifDimensions :: Get (Word32, Word32)
+findAvifDimensions = searchAvifBoxes []
+
+searchAvifBoxes :: [B.ByteString] -> Get (Word32, Word32)
+searchAvifBoxes path = do
+  isempty <- isEmpty
+  if isempty
+    then fail $ "No dimensions found. Searched: " ++ show (reverse path)
+    else do
+      boxSize <- getWord32be
+      boxType <- getByteString 4
+
+      let contentSize = fromIntegral boxSize - 8
+      let newPath = boxType : path
+
+      -- If it's a container box, search inside it
+      if isContainerBox boxType
+        then searchInsideBox contentSize newPath
+        else do
+          -- Try to parse dimensions from this box
+          result <- tryParseDimensions boxType contentSize
+          case result of
+            Just dims -> return dims
+            Nothing -> do
+              -- Skip this box and continue
+              when (contentSize > 0 && contentSize < 10000000) $
+                skip contentSize
+              searchAvifBoxes path
+
+tryParseDimensions :: B.ByteString -> Int -> Get (Maybe (Word32, Word32))
+tryParseDimensions boxType size = do
+  pos <- bytesRead
+  result <- case boxType of
+    "ispe" -> parseIspeBox
+    "tkhd" -> parseTkhdBox
+    "stsd" -> parseStsdBox
+    "av01" -> parseAv01Box
+    _ -> return Nothing
+
+  -- Reset position if we didn't find dimensions
+  case result of
+    Nothing -> do
+      newPos <- bytesRead
+      let consumed = fromIntegral (newPos - pos)
+      case size - consumed of
+        n | n > 0 -> skip n
+        _ -> return ()
+    Just _ -> return ()
+
+  return result
+
+parseIspeBox :: Get (Maybe (Word32, Word32))
+parseIspeBox = do
+  skip 4  -- version/flags
+  width <- getWord32be
+  height <- getWord32be
+  return $ Just (width, height)
+
+parseTkhdBox :: Get (Maybe (Word32, Word32))
+parseTkhdBox = do
+  version <- getWord8
+  skip 3  -- flags
+
+  -- Skip to width/height based on version
+  let skipBytes = if version == 1 then 76 else 64
+  skip skipBytes
+
+  width <- getWord32be
+  height <- getWord32be
+  -- Convert from 16.16 fixed point
+  return $ Just (width `shiftR` 16, height `shiftR` 16)
+
+parseStsdBox :: Get (Maybe (Word32, Word32))
+parseStsdBox = do
+  skip 8  -- version, flags, entry count
+  findAv01Entry
+
+findAv01Entry :: Get (Maybe (Word32, Word32))
+findAv01Entry = do
+  entrySize <- getWord32be
+  codec <- getByteString 4
+
+  if codec == "av01"
+    then do
+      skip 6  -- reserved
+      skip 2  -- data reference index
+      skip 16 -- pre-defined + reserved
+      width <- getWord16be
+      height <- getWord16be
+      return $ Just (fromIntegral width, fromIntegral height)
+    else do
+      let skipSize = fromIntegral entrySize - 8
+      when (skipSize > 0) $ skip skipSize
+      findAv01Entry
+
+parseAv01Box :: Get (Maybe (Word32, Word32))
+parseAv01Box = do
+  skip 6   -- reserved
+  skip 2   -- data reference index
+  skip 16  -- predefined/reserved
+  width <- getWord16be
+  height <- getWord16be
+  return $ Just (fromIntegral width, fromIntegral height)
+
+searchInsideBox :: Int -> [B.ByteString] -> Get (Word32, Word32)
+searchInsideBox size path = do
+  -- For meta boxes, skip version/flags
+  let isMeta = case path of
+                 "meta":_ -> True
+                 _ -> False
+  when isMeta $ skip 4
+
+  let searchSize = if isMeta then size - 4 else size
+  searchAvifBoxesInRange searchSize path
+
+searchAvifBoxesInRange :: Int -> [B.ByteString] -> Get (Word32, Word32)
+searchAvifBoxesInRange remaining' path
+  | remaining' < 8 = searchAvifBoxes path
+  | otherwise = do
+      boxSize <- getWord32be
+      boxType <- getByteString 4
+
+      let contentSize = fromIntegral boxSize - 8
+      let newPath = boxType : path
+
+      when (contentSize < 0 || fromIntegral boxSize > remaining') $ do
+        fail $ "Malformed box at path: " ++ show (reverse newPath)
+
+      if isContainerBox boxType
+        then searchInsideBox contentSize newPath
+        else do
+          result <- tryParseDimensions boxType contentSize
+          case result of
+            Just dims -> return dims
+            Nothing -> do
+              -- Don't skip here - tryParseDimensions already handled it
+              searchAvifBoxesInRange (remaining' - fromIntegral boxSize) path
+
+isContainerBox :: B.ByteString -> Bool
+isContainerBox boxType = boxType `elem`
+  ["moov", "trak", "mdia", "minf", "stbl", "meta", "dinf", "ipco", "iprp"]
