@@ -3,10 +3,11 @@
 {-# LANGUAGE DeriveGeneric      #-}
 {-# LANGUAGE LambdaCase         #-}
 {-# LANGUAGE OverloadedStrings  #-}
+{-# LANGUAGE PatternSynonyms    #-}
 {-# LANGUAGE TemplateHaskell    #-}
 {- |
    Module      : Text.Pandoc.Options
-   Copyright   : Copyright (C) 2012-2022 John MacFarlane
+   Copyright   : Copyright (C) 2012-2024 John MacFarlane
    License     : GNU GPL, version 2 or above
 
    Maintainer  : John MacFarlane <jgm@berkeley.edu>
@@ -21,6 +22,10 @@ module Text.Pandoc.Options ( module Text.Pandoc.Extensions
                            , HTMLMathMethod (..)
                            , CiteMethod (..)
                            , ObfuscationMethod (..)
+                           , HighlightMethod (..)
+                           , pattern NoHighlightingString
+                           , pattern DefaultHighlightingString
+                           , pattern IdiomaticHighlightingString
                            , HTMLSlideVariant (..)
                            , EPUBVersion (..)
                            , WrapOption (..)
@@ -28,9 +33,11 @@ module Text.Pandoc.Options ( module Text.Pandoc.Extensions
                            , WriterOptions (..)
                            , TrackChanges (..)
                            , ReferenceLocation (..)
+                           , CaptionPosition (..)
                            , def
                            , isEnabled
                            , defaultMathJaxURL
+                           , defaultWebTeXURL
                            , defaultKaTeXURL
                            ) where
 import Control.Applicative ((<|>))
@@ -40,12 +47,14 @@ import Data.Default
 import Data.Char (toLower)
 import Data.Text (Text)
 import qualified Data.Set as Set
+import qualified Data.Text as T
 import Data.Typeable (Typeable)
 import GHC.Generics (Generic)
 import Skylighting (SyntaxMap, defaultSyntaxMap)
 import Text.DocTemplates (Context(..), Template)
 import Text.Pandoc.Extensions
-import Text.Pandoc.Highlighting (Style, pygments)
+import Text.Pandoc.Chunks (PathTemplate)
+import Text.Pandoc.Highlighting (Style)
 import Text.Pandoc.UTF8 (toStringLazy)
 import Data.Aeson.TH (deriveJSON)
 import Data.Aeson
@@ -112,7 +121,8 @@ instance FromJSON HTMLMathMethod where
         mburl <- m .:? "url"
         case method :: Text of
           "plain" -> return PlainMath
-          "webtex" -> return $ WebTeX $ fromMaybe "" mburl
+          "webtex" -> return $ WebTeX $
+                         fromMaybe defaultWebTeXURL mburl
           "gladtex" -> return GladTeX
           "mathml" -> return MathML
           "mathjax" -> return $ MathJax $
@@ -122,7 +132,7 @@ instance FromJSON HTMLMathMethod where
           _ -> fail $ "Unknown HTML math method " ++ show method) node
        <|> (case node of
                String "plain" -> return PlainMath
-               String "webtex" -> return $ WebTeX ""
+               String "webtex" -> return $ WebTeX defaultWebTeXURL
                String "gladtex" -> return GladTeX
                String "mathml" -> return MathML
                String "mathjax" -> return $ MathJax defaultMathJaxURL
@@ -182,6 +192,43 @@ instance ToJSON ObfuscationMethod where
    toJSON ReferenceObfuscation = String "references"
    toJSON JavascriptObfuscation = String "javascript"
 
+-- | Method to provide code highlighting.
+data HighlightMethod =
+    Skylighting Style
+  | IdiomaticHighlighting
+  | DefaultHighlighting
+  | NoHighlighting
+  deriving (Show, Read, Eq, Data, Typeable, Generic)
+
+-- | String representation of the idiomatic highlighting option.
+pattern IdiomaticHighlightingString :: Text
+pattern IdiomaticHighlightingString = "idiomatic"
+
+-- | String representation of the default highlighting option.
+pattern DefaultHighlightingString :: Text
+pattern DefaultHighlightingString = "default"
+
+-- | String representation of the no highlighting option
+pattern NoHighlightingString :: Text
+pattern NoHighlightingString = "none"
+
+instance ToJSON HighlightMethod where
+  toJSON NoHighlighting        = String NoHighlightingString
+  toJSON IdiomaticHighlighting = String IdiomaticHighlightingString
+  toJSON DefaultHighlighting   = String DefaultHighlightingString
+  toJSON (Skylighting style)   = toJSON style
+
+instance FromJSON HighlightMethod where
+  parseJSON = \case
+    String NoHighlightingString        -> pure NoHighlighting
+    String IdiomaticHighlightingString -> pure IdiomaticHighlighting
+    String DefaultHighlightingString   -> pure DefaultHighlighting
+    String x           -> fail $ "Unknown highlighting method " <> T.unpack x
+    Bool True          -> pure DefaultHighlighting
+    Bool False         -> pure NoHighlighting
+    Null               -> pure NoHighlighting
+    v                  -> Skylighting <$> parseJSON v
+
 -- | Varieties of HTML slide shows.
 data HTMLSlideVariant = S5Slides
                       | SlidySlides
@@ -285,12 +332,30 @@ instance ToJSON ReferenceLocation where
    toJSON EndOfSection = "end-of-section"
    toJSON EndOfDocument = "end-of-document"
 
+-- | Positions for figure and table captions
+data CaptionPosition = CaptionAbove -- ^ above figure or table
+                     | CaptionBelow -- ^ below figure or table
+                     deriving (Show, Read, Eq, Data, Typeable, Generic)
+
+instance FromJSON CaptionPosition where
+  parseJSON v =
+    case v of
+      String "above" -> return CaptionAbove
+      String "below" -> return CaptionBelow
+      _ -> fail $ "Unknown caption position " <> toStringLazy (encode v)
+
+instance ToJSON CaptionPosition where
+   toJSON CaptionAbove = "above"
+   toJSON CaptionBelow = "below"
+
 -- | Options for writers
 data WriterOptions = WriterOptions
   { writerTemplate          :: Maybe (Template Text) -- ^ Template to use
   , writerVariables         :: Context Text -- ^ Variables to set in template
   , writerTabStop           :: Int    -- ^ Tabstop for conversion btw spaces and tabs
   , writerTableOfContents   :: Bool   -- ^ Include table of contents
+  , writerListOfFigures     :: Bool   -- ^ Include list of figures
+  , writerListOfTables      :: Bool   -- ^ Include list of tables
   , writerIncremental       :: Bool   -- ^ True if lists should be incremental
   , writerHTMLMathMethod    :: HTMLMathMethod  -- ^ How to print math in HTML
   , writerNumberSections    :: Bool   -- ^ Number sections in LaTeX
@@ -308,19 +373,23 @@ data WriterOptions = WriterOptions
   , writerHtmlQTags         :: Bool       -- ^ Use @<q>@ tags for quotes in HTML
   , writerSlideLevel        :: Maybe Int  -- ^ Force header level of slides
   , writerTopLevelDivision  :: TopLevelDivision -- ^ Type of top-level divisions
-  , writerListings          :: Bool       -- ^ Use listings package for code
-  , writerHighlightStyle    :: Maybe Style  -- ^ Style to use for highlighting
-                                           -- (Nothing = no highlighting)
+  , writerHighlightMethod   :: HighlightMethod  -- ^ Style to use for highlighting
   , writerSetextHeaders     :: Bool       -- ^ Use setext headers for levels 1-2 in markdown
+  , writerListTables        :: Bool       -- ^ Use list tables for RST tables
   , writerEpubSubdirectory  :: Text       -- ^ Subdir for epub in OCF
   , writerEpubMetadata      :: Maybe Text -- ^ Metadata to include in EPUB
   , writerEpubFonts         :: [FilePath] -- ^ Paths to fonts to embed
-  , writerEpubChapterLevel  :: Int            -- ^ Header level for chapters (separate files)
+  , writerEpubTitlePage     :: Bool           -- ^ Include title page in epub
+  , writerSplitLevel        :: Int        -- ^ Header level at which to split EPUB or chunked HTML into separate files
+  , writerChunkTemplate     :: PathTemplate  -- ^ Template for filenames in chunked HTML
   , writerTOCDepth          :: Int            -- ^ Number of levels to include in TOC
   , writerReferenceDoc      :: Maybe FilePath -- ^ Path to reference document if specified
   , writerReferenceLocation :: ReferenceLocation    -- ^ Location of footnotes and references for writing markdown
+  , writerFigureCaptionPosition :: CaptionPosition -- ^ Position of figure caption
+  , writerTableCaptionPosition :: CaptionPosition -- ^ Position of table caption
   , writerSyntaxMap         :: SyntaxMap
   , writerPreferAscii       :: Bool           -- ^ Prefer ASCII representations of characters when possible
+  , writerLinkImages        :: Bool           -- ^ Use links rather than embedding ODT images
   } deriving (Show, Data, Typeable, Generic)
 
 instance Default WriterOptions where
@@ -328,6 +397,8 @@ instance Default WriterOptions where
                       , writerVariables        = mempty
                       , writerTabStop          = 4
                       , writerTableOfContents  = False
+                      , writerListOfFigures    = False
+                      , writerListOfTables     = False
                       , writerIncremental      = False
                       , writerHTMLMathMethod   = PlainMath
                       , writerNumberSections   = False
@@ -344,18 +415,23 @@ instance Default WriterOptions where
                       , writerHtmlQTags        = False
                       , writerSlideLevel       = Nothing
                       , writerTopLevelDivision = TopLevelDefault
-                      , writerListings         = False
-                      , writerHighlightStyle   = Just pygments
+                      , writerHighlightMethod  = DefaultHighlighting
                       , writerSetextHeaders    = False
+                      , writerListTables       = False
                       , writerEpubSubdirectory = "EPUB"
                       , writerEpubMetadata     = Nothing
                       , writerEpubFonts        = []
-                      , writerEpubChapterLevel = 1
+                      , writerEpubTitlePage    = True
+                      , writerSplitLevel       = 1
+                      , writerChunkTemplate    = "%s-%i.html"
                       , writerTOCDepth         = 3
                       , writerReferenceDoc     = Nothing
                       , writerReferenceLocation = EndOfDocument
+                      , writerFigureCaptionPosition = CaptionBelow
+                      , writerTableCaptionPosition  = CaptionAbove
                       , writerSyntaxMap        = defaultSyntaxMap
                       , writerPreferAscii      = False
+                      , writerLinkImages       = False
                       }
 
 instance HasSyntaxExtensions WriterOptions where
@@ -368,8 +444,11 @@ isEnabled ext opts = ext `extensionEnabled` getExtensions opts
 defaultMathJaxURL :: Text
 defaultMathJaxURL = "https://cdn.jsdelivr.net/npm/mathjax@3/es5/tex-chtml-full.js"
 
+defaultWebTeXURL :: Text
+defaultWebTeXURL = "https://latex.codecogs.com/png.latex?"
+
 defaultKaTeXURL :: Text
-defaultKaTeXURL = "https://cdn.jsdelivr.net/npm/katex@0.15.1/dist/"
+defaultKaTeXURL = "https://cdn.jsdelivr.net/npm/katex@latest/dist/"
 
 -- Update documentation in doc/filters.md if this is changed.
 $(deriveJSON defaultOptions{ fieldLabelModifier =
