@@ -43,6 +43,7 @@ import qualified System.IO.Error as IE
 import Text.DocLayout (literal, render, hsep)
 import Text.Pandoc.Definition
 import Text.Pandoc.Error (PandocError (PandocPDFProgramNotFoundError))
+import Text.Pandoc.SelfContained (makeSelfContained)
 import Text.Pandoc.MIME (getMimeType)
 import Text.Pandoc.Options (HTMLMathMethod (..), WriterOptions (..))
 import Text.Pandoc.Extensions (disableExtension, Extension(Ext_smart))
@@ -57,7 +58,8 @@ import Control.Monad.Catch (MonadMask)
 import Data.List (intercalate)
 #endif
 import Data.List (isPrefixOf, find)
-import Text.Pandoc.Class (fillMediaBag, getVerbosity, setVerbosity,
+import Text.Pandoc.MediaBag (mediaItems)
+import Text.Pandoc.Class (fillMediaBag, getMediaBag, getVerbosity, setVerbosity,
                           readFileStrict, fileExists,
                           report, extractMedia, PandocMonad, runIOorExplode)
 import Text.Pandoc.Logging
@@ -81,24 +83,36 @@ makePDF :: (PandocMonad m, MonadIO m, MonadMask m)
         -> WriterOptions       -- ^ options
         -> Pandoc              -- ^ document
         -> m (Either ByteString ByteString)
-makePDF program pdfargs writer opts doc =
+makePDF program pdfargs writer opts doc = withTempDir (program == "typst") "media" $ \mediaDir -> do
+#ifdef _WINDOWS
+  -- note:  we want / even on Windows, for TexLive
+  let tmpdir = changePathSeparators mediaDir
+#else
+  let tmpdir = mediaDir
+#endif
+  doc' <- handleImages opts tmpdir doc
+  source <- writer opts{ writerExtensions = -- disable use of quote
+                            -- ligatures to avoid bad ligatures like ?`
+                            disableExtension Ext_smart
+                             (writerExtensions opts) } doc'
+  verbosity <- getVerbosity
+  let compileHTML mkOutArgs = do
+        -- check to see if there is anything in mediabag, and if so,
+        -- make the HTML self contained
+        mediabag <- getMediaBag
+        source' <- case mediaItems mediabag of
+                      [] -> pure source
+                      _ -> makeSelfContained source
+        liftIO $
+          toPdfViaTempFile verbosity program pdfargs mkOutArgs ".html" source'
   case takeBaseName program of
     "wkhtmltopdf" -> makeWithWkhtmltopdf program pdfargs writer opts doc
-    prog | prog `elem` ["pagedjs-cli" ,"weasyprint", "prince"] -> do
-      let mkOutArgs f =
-            if program `elem` ["pagedjs-cli", "prince"]
-               then ["-o", f]
-               else [f]
-      source <- writer opts doc
-      verbosity <- getVerbosity
-      liftIO $ toPdfViaTempFile verbosity program pdfargs mkOutArgs ".html" source
-    "typst" -> do
-      source <- writer opts doc
-      verbosity <- getVerbosity
-      liftIO $
+    "pagedjs-cli" -> compileHTML (\f -> ["-o", f])
+    "prince"      -> compileHTML (\f -> ["-o", f])
+    "weasyprint"  -> compileHTML (:[])
+    "typst" -> liftIO $
         toPdfViaTempFile verbosity program ("compile":pdfargs) (:[]) ".typ" source
     "pdfroff" -> do
-      source <- writer opts doc
       let paperargs =
             case lookupContext "papersize" (writerVariables opts) of
               Just s
@@ -112,7 +126,6 @@ makePDF program pdfargs writer opts doc =
                     paperargs ++ pdfargs
       generic2pdf program args source
     "groff" -> do
-      source <- writer opts doc
       let paperargs =
             case lookupContext "papersize" (writerVariables opts) of
               Just s
@@ -125,33 +138,22 @@ makePDF program pdfargs writer opts doc =
                    ["-U" | ".PDFPIC" `T.isInfixOf` source] ++
                     paperargs ++ pdfargs
       generic2pdf program args source
-    baseProg -> do
-      withTempDir "tex2pdf." $ \tmpdir' -> do
-#ifdef _WINDOWS
-        -- note:  we want / even on Windows, for TexLive
-        let tmpdir = changePathSeparators tmpdir'
-#else
-        let tmpdir = tmpdir'
-#endif
-        doc' <- handleImages opts tmpdir doc
-        source <- writer opts{ writerExtensions = -- disable use of quote
-                                  -- ligatures to avoid bad ligatures like ?`
-                                  disableExtension Ext_smart
-                                   (writerExtensions opts) } doc'
-        case baseProg of
-          "context" -> context2pdf program pdfargs tmpdir source
-          "tectonic" -> tectonic2pdf program pdfargs tmpdir source
-          prog | prog `elem` ["pdflatex", "lualatex", "xelatex", "latexmk"]
-              -> tex2pdf program pdfargs tmpdir source
-          _ -> return $ Left $ UTF8.fromStringLazy
-                             $ "Unknown program " ++ program
+    "context"      -> context2pdf program pdfargs tmpdir source
+    "tectonic"     -> tectonic2pdf program pdfargs tmpdir source
+    "latexmk"      -> tex2pdf program pdfargs tmpdir source
+    "lualatex"     -> tex2pdf program pdfargs tmpdir source
+    "lualatex-dev" -> tex2pdf program pdfargs tmpdir source
+    "pdflatex"     -> tex2pdf program pdfargs tmpdir source
+    "pdflatex-dev" -> tex2pdf program pdfargs tmpdir source
+    "xelatex"      -> tex2pdf program pdfargs tmpdir source
+    _ -> return $ Left $ UTF8.fromStringLazy $ "Unknown program " ++ program
 
 -- latex has trouble with tildes in paths, which
 -- you find in Windows temp dir paths with longer
 -- user names (see #777)
 withTempDir :: (PandocMonad m, MonadMask m, MonadIO m)
-            => FilePath -> (FilePath -> m a) -> m a
-withTempDir templ action = do
+            => Bool -> FilePath -> (FilePath -> m a) -> m a
+withTempDir useWorkingDirectory templ action = do
   tmp <- liftIO getTemporaryDirectory
   uname <- liftIO $ E.catch
     (do (ec, sout, _) <- readProcessWithExitCode "uname" ["-o"] ""
@@ -159,9 +161,9 @@ withTempDir templ action = do
            then return $ Just $ filter (not . isSpace) sout
            else return Nothing)
     (\(_  :: E.SomeException) -> return Nothing)
-  if '~' `elem` tmp || uname == Just "Cygwin" -- see #5451
-         then withTempDirectory "." templ action
-         else withSystemTempDirectory templ action
+  if useWorkingDirectory || '~' `elem` tmp || uname == Just "Cygwin" -- see #5451
+     then withTempDirectory "." templ action
+     else withSystemTempDirectory templ action
 
 makeWithWkhtmltopdf :: (PandocMonad m, MonadIO m)
                     => String              -- ^ wkhtmltopdf or path
@@ -328,7 +330,7 @@ latexWarnings log' = foldM_ go Nothing (BC.lines log')
    go (Just msg) ln
      | ln == "" = do -- emit report and reset accumulator
          report $ MakePDFWarning $ render (Just 60) $
-            hsep $ map literal $ T.words $ UTF8.toText $ BC.toStrict msg
+            hsep $ map literal $ T.words $ utf8ToText msg
          pure Nothing
      | otherwise = pure $ Just (msg <> ln)
 
@@ -372,7 +374,7 @@ runTectonic program args' tmpDir' source = do
     (exit, out) <- liftIO $ E.catch
       (pipeProcess (Just env) program programArgs sourceBL)
       (handlePDFProgramNotFound program)
-    report $ MakePDFInfo "tectonic output" (UTF8.toText $ BL.toStrict out)
+    report $ MakePDFInfo "tectonic output" (utf8ToText out)
     let pdfFile = tmpDir ++ "/texput.pdf"
     (_, pdf) <- getResultingPDF Nothing pdfFile
     return (exit, out, pdf)
@@ -435,7 +437,7 @@ runTeXProgram program args tmpDir outDir = do
      (exit, out) <- liftIO $ E.catch
        (pipeProcess (Just env'') program programArgs BL.empty)
        (handlePDFProgramNotFound program)
-     report $ MakePDFInfo "LaTeX output" (UTF8.toText $ BL.toStrict out)
+     report $ MakePDFInfo "LaTeX output" (utf8ToText out)
      -- parse log to see if we need to rerun LaTeX
      let logFile = replaceExtension outfile ".log"
      logExists <- fileExists logFile
@@ -449,8 +451,7 @@ runTeXProgram program args tmpDir outDir = do
      if not (null rerunWarnings') && runNumber < maxruns
         then do
           report $ MakePDFInfo "Rerun needed"
-                    (T.intercalate "\n"
-                      (map (UTF8.toText . BC.toStrict) rerunWarnings'))
+                    (T.intercalate "\n" (map utf8ToText rerunWarnings'))
           go env'' programArgs (runNumber + 1)
        else do
           (log', pdf) <- getResultingPDF (Just logFile) outfile
@@ -503,7 +504,7 @@ toPdfViaTempFile verbosity program args mkOutArgs extension source =
         (handlePDFProgramNotFound program)
       runIOorExplode $ do
         setVerbosity verbosity
-        report $ MakePDFInfo "pdf-engine output" (UTF8.toText $ BL.toStrict out)
+        report $ MakePDFInfo "pdf-engine output" (utf8ToText out)
       pdfExists <- doesFileExist pdfFile
       mbPdf <- if pdfExists
                 -- We read PDF as a strict bytestring to make sure that the
@@ -536,7 +537,7 @@ context2pdf program pdfargs tmpDir source = do
       (handlePDFProgramNotFound program)
     runIOorExplode $ do
       setVerbosity verbosity
-      report $ MakePDFInfo "ConTeXt run output" (UTF8.toText $ BL.toStrict out)
+      report $ MakePDFInfo "ConTeXt run output" (utf8ToText out)
     let pdfFile = replaceExtension file ".pdf"
     pdfExists <- doesFileExist pdfFile
     mbPdf <- if pdfExists
