@@ -305,7 +305,8 @@ runStyleToTransform rPr' = do
             emph . go rPr{isItalic = Nothing, isItalicCTL = Nothing}
         | Just True <- bold rPr =
             strong . go rPr{isBold = Nothing, isBoldCTL = Nothing}
-        | Just _ <- rHighlight rPr =
+        | Just v <- rHighlight rPr
+        , v /= "none" =
             spanWith ("",["mark"],[]) . go rPr{rHighlight = Nothing}
         | Just True <- isSmallCaps rPr =
             smallcaps . go rPr{isSmallCaps = Nothing}
@@ -800,14 +801,17 @@ bodyPartToBlocks (Captioned parstyle parparts bpart) = do
   bs <- bodyPartToBlocks bpart
   captContents <- bodyPartToBlocks (Paragraph parstyle parparts)
   let capt = Caption Nothing (toList captContents)
-  case toList bs of
-    [Table attr _cap colspecs thead tbodies tfoot]
-      -> pure $ singleton $ Table attr capt colspecs thead tbodies tfoot
-    [Figure attr _cap blks]
-      -> pure $ singleton $ Figure attr capt blks
-    [Para im@[Image{}]]
-      -> pure $ singleton $ Figure nullAttr capt [Plain im]
-    _ -> pure captContents
+  let toCaptioned attr' bls = case bls of
+        [Table attr _cap colspecs thead tbodies tfoot]
+          -> singleton $ Table (attr <> attr') capt colspecs thead tbodies tfoot
+        [Figure attr _cap blks]
+          -> singleton $ Figure (attr <> attr') capt blks
+        [Para im@[Image{}]]
+          -> singleton $ Figure attr' capt [Plain im]
+        [Div attr bls']
+          -> toCaptioned (attr <> attr') bls'
+        _ -> captContents
+  pure $ toCaptioned nullAttr (toList bs)
 bodyPartToBlocks (Tbl _ _ _ _ []) =
   return mempty
 bodyPartToBlocks (Tbl mbsty cap grid look parts) = do
@@ -816,8 +820,6 @@ bodyPartToBlocks (Tbl mbsty cap grid look parts) = do
       cap' = caption shortCaption fullCaption
       (hdr, rows) = splitHeaderRows (firstRowFormatting look) parts
 
-  let rowHeadCols = if firstColumnFormatting look then 1 else 0
-
   let width = maybe 0 maximum $ nonEmpty $ map rowLength parts
       rowLength :: Docx.Row -> Int
       rowLength (Docx.Row _ c) = sum (fmap (\(Docx.Cell _ gridSpan _ _) -> fromIntegral gridSpan) c)
@@ -841,7 +843,7 @@ bodyPartToBlocks (Tbl mbsty cap grid look parts) = do
   return $ tableWith attr cap'
                  (zip alignments widths)
                  (TableHead nullAttr headerCells)
-                 [TableBody nullAttr (RowHeadColumns rowHeadCols) [] bodyCells]
+                 [TableBody nullAttr 0 [] bodyCells]
                  (TableFoot nullAttr [])
 bodyPartToBlocks HRule = pure Pandoc.horizontalRule
 
