@@ -242,7 +242,7 @@ block = ((do
         "main" -> pDiv
         "figure" -> pFigure
         "iframe" -> pIframe
-        "style" -> pRawHtmlBlock
+        "style" -> mempty <$ pHtmlBlock "style" -- see #10643
         "textarea" -> pRawHtmlBlock
         "switch"
           | epubExts
@@ -537,7 +537,7 @@ pIframe = try $ do
 
 pRawHtmlBlock :: PandocMonad m => TagParser m Blocks
 pRawHtmlBlock = do
-  raw <- pHtmlBlock "script" <|> pHtmlBlock "style" <|> pHtmlBlock "textarea"
+  raw <- pHtmlBlock "script" <|> pHtmlBlock "textarea"
           <|> pRawTag
   exts <- getOption readerExtensions
   if extensionEnabled Ext_raw_html exts && not (T.null raw)
@@ -659,14 +659,10 @@ pCodeBlock = try $ do
                              else v ]
   contents <- manyTill pAny (pCloses "pre" <|> eof)
   let rawText = T.concat $ map tagToText contents
-  -- drop leading newline if any
-  let result' = case T.uncons rawText of
-                     Just ('\n', xs) -> xs
-                     _               -> rawText
   -- drop trailing newline if any
-  let result = case T.unsnoc result' of
+  let result = case T.unsnoc rawText of
                     Just (result'', '\n') -> result''
-                    _                     -> result'
+                    _                     -> rawText
   return $ B.codeBlockWith attr result
 
 tagToText :: Tag Text -> Text
@@ -716,6 +712,7 @@ inline = pTagText <|> do
         "input"
           | lookup "type" attr == Just "checkbox"
           -> asks inListItem >>= guard >> pCheckbox
+        "style" -> mempty <$ pHtmlBlock "style" -- see #10643
         "script"
           | Just x <- lookup "type" attr
           , "math/tex" `T.isPrefixOf` x -> pScriptMath
@@ -878,6 +875,11 @@ pSpan = do
   exts <- getOption readerExtensions
   let attr = toAttr attr'
   case attr of
+     (_,cls,_) -- skip MathJaX-generated HTML; see #10673
+       | "mjx-chtml" `elem` cls -> mempty <$ pInTags "span" inline
+       | "MathJax_CHTML" `elem` cls -> mempty <$ pInTags "span" inline
+       | "MathJax_Preview" `elem` cls -> mempty <$ pInTags "span" inline
+       | "MJX_Assistive_MathML" `elem` cls -> pInTags "span" inline
      (_,["katex-html"],_) -> mempty <$ pInTags "span" inline
        -- skip HTML generated by KaTeX, since we get
        -- the math by parsing mathml (#9971)
@@ -1075,6 +1077,8 @@ isInlineTag :: Tag Text -> Bool
 isInlineTag t = isCommentTag t || case t of
   TagOpen "script" _ -> "math/tex" `T.isPrefixOf` fromAttrib "type" t
   TagClose "script"  -> True
+  TagOpen "style" _  -> True -- see #10643, invalid but it happens
+  TagClose "style"   -> True
   TagOpen name _     -> isInlineTagName name
   TagClose name      -> isInlineTagName name
   _                  -> False
