@@ -1,19 +1,20 @@
 {-# LANGUAGE LambdaCase #-}
 {-# LANGUAGE OverloadedStrings #-}
 {- |
-   Module      : Text.Pandoc.Readers.Org
-   Copyright   : © 2019-2022 Albert Krewinkel
-   License     : GNU GPL, version 2 or above
+   Module      : Text.Pandoc.Readers.Jira
+   Copyright   : © 2019-2024 Albert Krewinkel
+   License     : GPL-2.0-or-later
 
-   Maintainer  : Albert Krewinkel <tarleb+pandoc@moltkeplatz.de>
+   Maintainer  : Albert Krewinkel <albert+pandoc@tarleb.com>
 
 Conversion of jira wiki formatted plain text to 'Pandoc' document.
 -}
 module Text.Pandoc.Readers.Jira ( readJira ) where
 
 import Control.Monad.Except (throwError)
-import Data.Text (Text, append, pack, singleton, unpack)
-import Text.HTML.TagSoup.Entity (lookupEntity)
+import Data.List (partition)
+import Data.Text (Text, append, pack, singleton)
+import Text.Pandoc.XML (lookupEntity)
 import Text.Jira.Parser (parse)
 import Text.Pandoc.Class.PandocMonad (PandocMonad (..))
 import Text.Pandoc.Builder hiding (cell)
@@ -76,7 +77,16 @@ toPandocCodeBlocks langMay params txt =
 -- | Create a pandoc @'Div'@ from a panel.
 toPandocDiv :: [Jira.Parameter] -> [Jira.Block] -> Blocks
 toPandocDiv params =
-  divWith ("", ["panel"], map paramToPair params) . foldMap jiraToPandocBlocks
+  let (titles, params') = partition ((== "title") . Jira.parameterKey) params
+      addTitle = case titles of
+        [] ->
+          id
+        (title:_) -> \blks ->
+          (divWith ("", ["panelheader"], []) . plain . strong $
+           text (Jira.parameterValue title)) <> blks
+  in divWith ("", ["panel"], map paramToPair params')
+     . addTitle
+     . foldMap jiraToPandocBlocks
 
 paramToPair :: Jira.Parameter -> (Text, Text)
 paramToPair (Jira.Parameter key value) = (key, value)
@@ -137,9 +147,9 @@ jiraToPandocInlines = \case
   Jira.Styled style inlns -> fromStyle style $ fromInlines inlns
   where
     fromInlines  = foldMap jiraToPandocInlines
-    fromEntity e = case lookupEntity (unpack e ++ ";") of
+    fromEntity e = case lookupEntity (e <> ";") of
                      Nothing -> "&" `append` e `append` ";"
-                     Just cs -> pack cs
+                     Just t ->t
 
     fromStyle = \case
       Jira.Emphasis    -> emph
