@@ -52,8 +52,8 @@ import Text.Pandoc.Parsing hiding (blankline, many, mathDisplay, mathInline,
 import Text.Pandoc.TeX (Tok (..), TokType (..))
 import Text.Pandoc.Readers.LaTeX.Parsing
 import Text.Pandoc.Readers.LaTeX.Citation (citationCommands, cites)
-import Text.Pandoc.Readers.LaTeX.Math (dollarsMath, inlineEnvironments,
-                                       inlineEnvironment,
+import Text.Pandoc.Readers.LaTeX.Math (withMathMode, dollarsMath,
+                                       inlineEnvironments, inlineEnvironment,
                                        mathDisplay, mathInline,
                                        newtheorem, theoremstyle, proof,
                                        theoremEnvironment)
@@ -371,9 +371,11 @@ inlineCommands = M.unions
     , ("hbox", rawInlineOr "hbox" $ processHBox <$> tok)
     , ("vbox", rawInlineOr "vbox" tok)
     , ("lettrine", rawInlineOr "lettrine" lettrine)
-    , ("(", mathInline . untokenize <$> manyTill anyTok (controlSeq ")"))
-    , ("[", mathDisplay . untokenize <$> manyTill anyTok (controlSeq "]"))
-    , ("ensuremath", mathInline . untokenize <$> braced)
+    , ("(", withMathMode
+        (mathInline . untokenize <$> manyTill anyTok (controlSeq ")")))
+    , ("[", withMathMode
+        (mathDisplay . untokenize <$> manyTill anyTok (controlSeq "]")))
+    , ("ensuremath", withMathMode (mathInline . untokenize <$> braced))
     , ("texorpdfstring", const <$> tok <*> tok)
     -- old TeX commands
     , ("em", extractSpaces emph <$> inlines)
@@ -961,6 +963,7 @@ blockCommands = M.fromList
    , ("paragraph*", section ("",["unnumbered"],[]) 4)
    , ("subparagraph", section nullAttr 5)
    , ("subparagraph*", section ("",["unnumbered"],[]) 5)
+   , ("minisec", section ("",["unnumbered","unlisted"],[]) 6) -- from KOMA
    -- beamer slides
    , ("frametitle", section nullAttr 3)
    , ("framesubtitle", section nullAttr 4)
@@ -1035,8 +1038,8 @@ environments = M.union (tableEnvironments block inline) $
    , ("letter", env "letter" letterContents)
    , ("minipage", divWith ("",["minipage"],[]) <$>
        env "minipage" (skipopts *> spaces *> optional braced *> spaces *> blocks))
-   , ("figure", env "figure" $ skipopts *> figure')
-   , ("figure*", env "figure*" $ skipopts *> figure')
+   , ("figure", env "figure" figure')
+   , ("figure*", env "figure*" figure')
    , ("subfigure", env "subfigure" $ skipopts *> tok *> figure')
    , ("center", divWith ("", ["center"], []) <$> env "center" blocks)
    , ("quote", blockQuote <$> env "quote" blocks)
@@ -1208,15 +1211,18 @@ letterContents = do
 
 figure' :: PandocMonad m => LP m Blocks
 figure' = try $ do
+  sp
+  poshint <- option "" $ untokenize <$> bracketedToks
+  sp
   resetCaption
   innerContent <- many $ try (Left <$> label) <|> (Right <$> block)
   let content = walk go $ mconcat $ snd $ partitionEithers innerContent
   st <- getState
   let caption' = fromMaybe B.emptyCaption $ sCaption st
   let mblabel  = sLastLabel st
-  let attr     = case mblabel of
-                   Just lab -> (lab, [], [])
-                   Nothing  -> nullAttr
+  let kvs = [("latex-placement", poshint) | not (T.null poshint)]
+  let ident = fromMaybe "" mblabel
+  let attr  = (ident, [], kvs)
   case mblabel of
     Nothing   -> pure ()
     Just lab  -> do
