@@ -2,7 +2,7 @@
 {-# LANGUAGE ViewPatterns          #-}
 {- |
    Module      : Text.Pandoc.Readers.LaTeX.Inline
-   Copyright   : Copyright (C) 2006-2022 John MacFarlane
+   Copyright   : Copyright (C) 2006-2024 John MacFarlane
    License     : GNU GPL, version 2 or above
 
    Maintainer  : John MacFarlane <jgm@berkeley.edu>
@@ -14,6 +14,7 @@ module Text.Pandoc.Readers.LaTeX.Inline
   , verbCommands
   , charCommands
   , accentCommands
+  , miscCommands
   , nameCommands
   , biblatexInlineCommands
   , refCommands
@@ -27,10 +28,11 @@ import Data.Text (Text)
 import qualified Data.Text as T
 import Text.Pandoc.Builder
 import Text.Pandoc.Shared (toRomanNumeral, safeRead)
-import Text.Pandoc.Readers.LaTeX.Types (Tok (..), TokType (..))
+import Text.Pandoc.TeX (Tok (..), TokType (..))
 import Control.Applicative (optional, (<|>))
 import Control.Monad (guard, mzero, mplus, unless)
-import Text.Pandoc.Class.PandocMonad (PandocMonad (..), translateTerm)
+import Text.Pandoc.Class.PandocMonad (PandocMonad (..))
+import Text.Pandoc.Translations (translateTerm)
 import Text.Pandoc.Readers.LaTeX.Parsing
 import Text.Pandoc.Extensions (extensionEnabled, Extension(..))
 import Text.Pandoc.Parsing (getOption, updateState, getState, notFollowedBy,
@@ -182,6 +184,43 @@ verbCommands = M.fromList
   , ("Verb", doverb)
   ]
 
+miscCommands :: PandocMonad m => M.Map Text (LP m Inlines)
+miscCommands =
+  M.fromList
+  [ ("pounds", lit "£")
+  , ("euro", lit "€")
+  , ("copyright", lit "©")
+  , ("textasciicircum", lit "^")
+  , ("textasciitilde", lit "~")
+  , ("textbaht", lit "฿")
+  , ("textblank", lit "␢")
+  , ("textbigcircle", lit "○")
+  , ("textbrokenbar", lit "¦")
+  , ("textbullet", lit "•")
+  , ("textcentoldstyle", lit "¢")
+  , ("textcopyright", lit "©")
+  , ("textdagger", lit "†")
+  , ("textdegree", lit "°")
+  , ("textdollar", lit "$")
+  , ("textdong", lit "₫")
+  , ("textlira", lit "₤")
+  , ("textmu", lit "μ")
+  , ("textmusicalnote", lit "♪")
+  , ("textonehalf", lit "½")
+  , ("textonequarter", lit "¼")
+  , ("textparagraph", lit "¶")
+  , ("textpertenthousand", lit "‱")
+  , ("textpeso", lit "₱")
+  , ("textquotesingle", lit "'")
+  , ("textregistered", lit "®")
+  , ("textsection", lit "§")
+  , ("textsterling", lit "£")
+  , ("textthreequarters", lit "¾")
+  , ("textthreesuperior", lit "³")
+  , ("texttwosuperior", lit "²")
+  , ("textyen", lit "¥")
+  ]
+
 accentCommands :: PandocMonad m => LP m Inlines -> M.Map Text (LP m Inlines)
 accentCommands tok =
   let accent = accentWith tok
@@ -197,11 +236,6 @@ accentCommands tok =
   , ("AE", lit "Æ")
   , ("oe", lit "œ")
   , ("OE", lit "Œ")
-  , ("pounds", lit "£")
-  , ("euro", lit "€")
-  , ("copyright", lit "©")
-  , ("textasciicircum", lit "^")
-  , ("textasciitilde", lit "~")
   , ("H", accent '\779' Nothing) -- hungarumlaut
   , ("`", accent '\768' (Just '`')) -- grave
   , ("'", accent '\769' (Just '\'')) -- acute
@@ -247,6 +281,7 @@ charCommands = M.fromList
   , ("_", lit "_")
   , ("{", lit "{")
   , ("}", lit "}")
+  , ("-", lit "\x00ad") -- soft hyphen
   , ("qed", lit "\a0\x25FB")
   , ("lq", return (str "‘"))
   , ("rq", return (str "’"))
@@ -336,10 +371,11 @@ refCommands :: PandocMonad m => M.Map Text (LP m Inlines)
 refCommands = M.fromList
   [ ("label", rawInlineOr "label" dolabel)
   , ("ref", rawInlineOr "ref" $ doref "ref")
-  , ("cref", rawInlineOr "cref" $ doref "ref")       -- from cleveref.sty
-  , ("vref", rawInlineOr "vref" $ doref "ref+page")  -- from varioref.sty
+  , ("cref", rawInlineOr "cref" $ doref "ref+label")       -- from cleveref.sty
+  , ("Cref", rawInlineOr "Cref" $ doref "ref+Label")       -- from cleveref.sty
+  , ("vref", rawInlineOr "vref" $ doref "ref")  -- from varioref.sty
   , ("eqref", rawInlineOr "eqref" $ doref "eqref")   -- from amsmath.sty
-  , ("autoref", rawInlineOr "autoref" $ doref "autoref") -- from hyperref.sty
+  , ("autoref", rawInlineOr "autoref" $ doref "ref+label") -- from hyperref.sty
   ]
 
 acronymCommands :: PandocMonad m => M.Map Text (LP m Inlines)
@@ -394,5 +430,3 @@ doAcronymPlural form = do
   return . mconcat $ [spanWith ("",[],[("acronym-label", untokenize acro),
     ("acronym-form", "plural+" <> form)]) $
    mconcat [str $ untokenize acro, plural]]
-
-
