@@ -1,7 +1,9 @@
 {-# LANGUAGE OverloadedStrings #-}
 module Text.Pandoc.Readers.LaTeX.Math
-  ( dollarsMath
+  ( withMathMode
+  , dollarsMath
   , inlineEnvironments
+  , inlineEnvironmentNames
   , inlineEnvironment
   , mathInline
   , mathDisplay
@@ -11,12 +13,13 @@ module Text.Pandoc.Readers.LaTeX.Math
   , proof
   )
 where
-import Data.Maybe (fromMaybe)
+import Data.Maybe (fromMaybe, mapMaybe, listToMaybe)
+import Data.List (foldl')
 import Text.Pandoc.Walk (walk)
 import Text.Pandoc.Builder as B
 import qualified Data.Sequence as Seq
 import Text.Pandoc.Readers.LaTeX.Parsing
-import Text.Pandoc.Readers.LaTeX.Types
+import Text.Pandoc.TeX
 import Text.Pandoc.Class
 import Text.Pandoc.Shared (trimMath, stripTrailingNewlines)
 import Text.Pandoc.Parsing hiding (blankline, mathDisplay, mathInline,
@@ -26,11 +29,19 @@ import Control.Monad (guard, mzero)
 import qualified Data.Map as M
 import Data.Text (Text)
 
+withMathMode :: PandocMonad m => LP m a -> LP m a
+withMathMode p = do
+  oldMathMode <- sMathMode <$> getState
+  updateState $ \s -> s{ sMathMode = True }
+  result <- p
+  updateState $ \s -> s{ sMathMode = oldMathMode }
+  return result
+
 dollarsMath :: PandocMonad m => LP m Inlines
 dollarsMath = do
   symbol '$'
   display <- option False (True <$ symbol '$')
-  (do contents <- try $ untokenize <$> pDollarsMath 0
+  (do contents <- try $ untokenize <$> withMathMode (pDollarsMath 0)
       if display
          then mathDisplay contents <$ symbol '$'
          else return $ mathInline contents)
@@ -65,11 +76,10 @@ mathEnvWith f innerEnv name = f . mathDisplay . inner <$> mathEnv name
    where inner x = case innerEnv of
                         Nothing -> x
                         Just y  -> "\\begin{" <> y <> "}\n" <> x <>
-                                   "\\end{" <> y <> "}"
+                                   "\n\\end{" <> y <> "}"
 
 mathEnv :: PandocMonad m => Text -> LP m Text
-mathEnv name = do
-  skipopts
+mathEnv name = withMathMode $ do
   optional blankline
   res <- manyTill anyTok (end_ name)
   return $ stripTrailingNewlines $ untokenize res
@@ -80,22 +90,29 @@ inlineEnvironment = try $ do
   name <- untokenize <$> braced
   M.findWithDefault mzero name inlineEnvironments
 
+inlineEnvironmentNames :: [Text]
+inlineEnvironmentNames =
+  M.keys (inlineEnvironments :: M.Map Text (LP PandocPure Inlines))
+
 inlineEnvironments :: PandocMonad m => M.Map Text (LP m Inlines)
 inlineEnvironments = M.fromList [
     ("displaymath", mathEnvWith id Nothing "displaymath")
   , ("math", math <$> mathEnv "math")
-  , ("equation", mathEnvWith id Nothing "equation")
-  , ("equation*", mathEnvWith id Nothing "equation*")
-  , ("gather", mathEnvWith id (Just "gathered") "gather")
-  , ("gather*", mathEnvWith id (Just "gathered") "gather*")
-  , ("multline", mathEnvWith id (Just "gathered") "multline")
-  , ("multline*", mathEnvWith id (Just "gathered") "multline*")
-  , ("eqnarray", mathEnvWith id (Just "aligned") "eqnarray")
-  , ("eqnarray*", mathEnvWith id (Just "aligned") "eqnarray*")
-  , ("align", mathEnvWith id (Just "aligned") "align")
-  , ("align*", mathEnvWith id (Just "aligned") "align*")
-  , ("alignat", mathEnvWith id (Just "aligned") "alignat")
-  , ("alignat*", mathEnvWith id (Just "aligned") "alignat*")
+  , ("equation", mathEnvWith id (Just "equation") "equation")
+  , ("equation*", mathEnvWith id (Just "equation*") "equation*")
+  , ("gather", mathEnvWith id (Just "gather") "gather")
+  , ("gather*", mathEnvWith id (Just "gather*") "gather*")
+  , ("multline", mathEnvWith id (Just "multline") "multline")
+  , ("multline*", mathEnvWith id (Just "multline*") "multline*")
+  , ("eqnarray", mathEnvWith id (Just "eqnarray") "eqnarray")
+  , ("eqnarray*", mathEnvWith id (Just "eqnarray*") "eqnarray*")
+  , ("align", mathEnvWith id (Just "align") "align")
+  , ("align*", mathEnvWith id (Just "align*") "align*")
+  , ("alignat", mathEnvWith id (Just "alignat") "alignat")
+  , ("alignat*", mathEnvWith id (Just "alignat*") "alignat*")
+  , ("flalign", mathEnvWith id (Just "flalign") "flalign")
+  , ("flalign*", mathEnvWith id (Just "flalign*") "flalign*")
+  -- the following are not yet handled by texmath, so we use substitutes:
   , ("dmath", mathEnvWith id Nothing "dmath")
   , ("dmath*", mathEnvWith id Nothing "dmath*")
   , ("dgroup", mathEnvWith id (Just "aligned") "dgroup")
@@ -140,6 +157,16 @@ newtheorem inline = do
                             M.insert name spec tmap }
   return mempty
 
+extractLabelFromBlock :: Block -> Maybe Text
+extractLabelFromBlock (Para inlines) = extractLabel Nothing inlines
+  where
+    extractLabel = foldl' go
+    go :: Maybe Text -> Inline -> Maybe Text
+    go (Just t) _ = Just t
+    go Nothing (Span (_, _, attrs) _) = lookup "label" attrs
+    go Nothing _ = Nothing
+extractLabelFromBlock _ = Nothing
+
 theoremEnvironment :: PandocMonad m
                    => LP m Blocks -> LP m Inlines -> Text -> LP m Blocks
 theoremEnvironment blocks opt name = do
@@ -150,7 +177,7 @@ theoremEnvironment blocks opt name = do
     Just tspec -> do
        optTitle <- option mempty $ (\x -> space <> "(" <> x <> ")") <$> opt
        bs <- env name blocks
-       mblabel <- sLastLabel <$> getState
+       let mblabel = listToMaybe $ mapMaybe extractLabelFromBlock (toList bs)
 
        number <-
          if theoremNumber tspec
@@ -214,9 +241,8 @@ addQed bs =
   qedSign = B.str "\xa0\x25FB"
 
 italicize :: Block -> Block
-italicize x@(Para [Image{}]) = x -- see #6925
+italicize x@(Para [Image{}])  = x -- see #6925
+italicize x@(Plain [Image{}]) = x -- ditto
 italicize (Para ils) = Para [Emph ils]
 italicize (Plain ils) = Plain [Emph ils]
 italicize x = x
-
-
