@@ -31,6 +31,7 @@ import Text.Pandoc.Walk (query)
 import Text.Pandoc.Readers.Roff  -- TODO explicit imports
 import qualified Text.Pandoc.Parsing as P
 import qualified Data.Foldable as Foldable
+import Text.Pandoc.Shared (extractSpaces)
 
 data ManState = ManState { readerOptions   :: ReaderOptions
                          , metadata        :: Meta
@@ -218,7 +219,16 @@ parseTitle = do
   (ControlLine _ args _) <- mmacro "TH"
   let adjustMeta =
        case args of
-         (x:y:z:_) -> setMeta "title" (linePartsToInlines x) .
+         (x:y:z:a:b:_) -> setMeta "title" (linePartsToInlines x) .
+                          setMeta "section" (linePartsToInlines y) .
+                          setMeta "date" (linePartsToInlines z) .
+                          setMeta "footer" (linePartsToInlines a) .
+                          setMeta "header" (linePartsToInlines b)
+         [x,y,z,a] -> setMeta "title" (linePartsToInlines x) .
+                      setMeta "section" (linePartsToInlines y) .
+                      setMeta "date" (linePartsToInlines z) .
+                      setMeta "footer" (linePartsToInlines a)
+         [x,y,z]   -> setMeta "title" (linePartsToInlines x) .
                       setMeta "section" (linePartsToInlines y) .
                       setMeta "date" (linePartsToInlines z)
          [x,y]     -> setMeta "title" (linePartsToInlines x) .
@@ -241,12 +251,12 @@ linePartsToInlines = go False
     | otherwise = text s <> go mono xs
   go mono (Font fs: xs)
     | litals > 0 && litals >= lbolds && litals >= lmonos
-       = emph (go mono (Font fs{ fontItalic = False } :
+       = extractSpaces emph (go mono (Font fs{ fontItalic = False } :
                    map (adjustFontSpec (\s -> s{ fontItalic = False }))
                    itals)) <>
             go mono italsrest
     | lbolds > 0 && lbolds >= lmonos
-       = strong (go mono (Font fs{ fontBold = False } :
+       = extractSpaces strong (go mono (Font fs{ fontBold = False } :
               map (adjustFontSpec (\s -> s{ fontBold = False }))
               bolds)) <>
             go mono boldsrest
@@ -305,12 +315,12 @@ handleInlineMacro mname args _pos =
     "RI" -> parseAlternatingFonts [id, emph] args
     "BR" -> parseAlternatingFonts [strong, id] args
     "RB" -> parseAlternatingFonts [id, strong] args
-    "SY" -> return $ strong $ mconcat $ intersperse B.space
+    "SY" -> return $ extractSpaces strong $ mconcat $ intersperse B.space
                    $ map linePartsToInlines args
     "YS" -> return mempty
     "OP" -> case args of
               (x:ys) -> return $ B.space <> str "[" <> B.space <>
-                         mconcat (strong (linePartsToInlines x) :
+                         mconcat (extractSpaces strong (linePartsToInlines x) :
                            map ((B.space <>) . linePartsToInlines) ys)
                          <> B.space <> str "]"
               []     -> return mempty
@@ -319,16 +329,18 @@ handleInlineMacro mname args _pos =
 parseBold :: PandocMonad m => [Arg] -> ManParser m Inlines
 parseBold [] = do
   TextLine lparts <- mline
-  return $ strong $ linePartsToInlines lparts
+  return $ extractSpaces strong $ linePartsToInlines lparts
 parseBold args = return $
-  strong $ mconcat $ intersperse B.space $ map linePartsToInlines args
+  extractSpaces strong $
+  mconcat $ intersperse B.space $ map linePartsToInlines args
 
 parseItalic :: PandocMonad m => [Arg] -> ManParser m Inlines
 parseItalic [] = do
   TextLine lparts <- mline
-  return $ emph $ linePartsToInlines lparts
+  return $ extractSpaces emph $ linePartsToInlines lparts
 parseItalic args = return $
-  emph $ mconcat $ intersperse B.space $ map linePartsToInlines args
+  extractSpaces emph $
+  mconcat $ intersperse B.space $ map linePartsToInlines args
 
 parseAlternatingFonts :: [Inlines -> Inlines]
                       -> [Arg]
@@ -406,7 +418,16 @@ parseHeader = do
   return $ header lvl contents
 
 parseBlockQuote :: PandocMonad m => ManParser m Blocks
-parseBlockQuote = blockQuote <$> continuation
+parseBlockQuote = blockQuote <$>
+   (  (mmacro "RS" *> (mconcat <$> manyTill parseBlock (endmacro "RE")))
+  <|> parseIndentedParagraphs
+   )
+ where
+  parseIndentedParagraphs = try $ do
+    bareIP
+    first <- parsePara <|> parseCodeBlock
+    rest <- many $ try (memptyLine *> (parsePara <|> parseCodeBlock))
+    pure (first <> mconcat rest)
 
 data ListType = Ordered ListAttributes
               | Bullet
@@ -457,10 +478,8 @@ parseList = try $ do
 
 continuation :: PandocMonad m => ManParser m Blocks
 continuation =
-      mconcat <$> (mmacro "RS" *> manyTill parseBlock (endmacro "RE"))
-  <|> mconcat <$> many1 (  try (bareIP *> parsePara)
-                       <|> try (bareIP *> parseCodeBlock)
-                        )
+      (mmacro "RS" *> (mconcat <$> manyTill parseBlock (endmacro "RE")))
+  <|> try ((memptyLine <|> bareIP) *> (parsePara <|> parseCodeBlock))
 
 definitionListItem :: PandocMonad m
                    => ManParser m (Inlines, [Blocks])
@@ -473,11 +492,10 @@ definitionListItem = try $ do
                  mmacro "TQ"
                  parseInline
   skipMany memptyLine
-  inls <- option mempty parseInlines
-  skipMany memptyLine
-  continuations <- mconcat <$> many continuation
+  firstBlock <- parseBlock
+  otherBlocks <- mconcat <$> many continuation
   return ( mconcat (intersperse B.linebreak (term:moreterms))
-         , [para inls <> continuations])
+         , [firstBlock <> otherBlocks])
 
 parseDefinitionList :: PandocMonad m => ManParser m Blocks
 parseDefinitionList = definitionList <$> many1 definitionListItem
