@@ -2,10 +2,10 @@
 {-# LANGUAGE OverloadedStrings #-}
 {- |
    Module      : Text.Pandoc.Readers.Org.Inlines
-   Copyright   : Copyright (C) 2014-2022 Albert Krewinkel
-   License     : GNU GPL, version 2 or above
+   Copyright   : Copyright (C) 2014-2025 Albert Krewinkel
+   License     : GPL-2.0-or-later
 
-   Maintainer  : Albert Krewinkel <tarleb+pandoc@moltkeplatz.de>
+   Maintainer  : Albert Krewinkel <albert+pandoc@tarleb.com>
 
 Parsers for Org-mode inline elements.
 -}
@@ -42,10 +42,6 @@ import qualified Data.Text as T
 --
 -- Functions acting on the parser state
 --
-recordAnchorId :: PandocMonad m => Text -> OrgParser m ()
-recordAnchorId i = updateState $ \s ->
-  s{ orgStateAnchorIds = i : orgStateAnchorIds s }
-
 pushToInlineCharStack :: PandocMonad m => Char -> OrgParser m ()
 pushToInlineCharStack c = updateState $ \s ->
   s{ orgStateEmphasisCharStack = c:orgStateEmphasisCharStack s }
@@ -92,17 +88,18 @@ inline =
          , inlineCodeBlock
          , str
          , endline
+         , subscript   -- takes precedence over underlined text
+         , superscript
          , emphasizedText
          , code
          , math
          , displayMath
          , verbatim
-         , subscript
-         , superscript
          , inlineLaTeX
          , exportSnippet
          , macro
-         , smart
+         , smartQuotes
+         , specialStrings
          , symbol
          ] <* (guard =<< newlinesCountWithinLimits)
   <?> "inline"
@@ -126,8 +123,12 @@ linebreak :: PandocMonad m => OrgParser m (F Inlines)
 linebreak = try $ pure B.linebreak <$ string "\\\\" <* skipSpaces <* newline
 
 str :: PandocMonad m => OrgParser m (F Inlines)
-str = return . B.str <$> many1Char (noneOf $ specialChars ++ "\n\r ")
+str = return . B.str <$>
+      ( many1Char (noneOf $ specialChars ++ "\n\r ") >>= updatePositions' )
       <* updateLastStrPos
+  where
+    updatePositions' str' = str' <$
+      maybe mzero (updatePositions . snd) (T.unsnoc str')
 
 -- | An endline character that can be treated as a space, not a structural
 -- break.  This should reflect the values of the Emacs variable
@@ -199,10 +200,10 @@ addSuffixToLastItem aff cs = do
                                 citationSuffix d <> B.toList aff' }])
 
 citeItems :: PandocMonad m => OrgParser m (F [Citation])
-citeItems = sequence <$> citeItem `sepBy1` (char ';')
+citeItems = sequence <$> sepBy1' citeItem (char ';' <* void (many spaceChar))
 
 citeItem :: PandocMonad m => OrgParser m (F Citation)
-citeItem = do
+citeItem = try $ do
   pref <- citePrefix
   itemKey <- orgCiteKey
   suff <- citeSuffix
@@ -246,37 +247,36 @@ citePrefix =
 
 citeSuffix :: PandocMonad m => OrgParser m (F Inlines)
 citeSuffix =
-  rawAffix False >>= parseFromString parseSuffix
- where
-   parseSuffix = do
-     hasSpace <- option False
-              (True <$ try (spaceChar >> skipSpaces >> lookAhead nonspaceChar))
-     ils <- trimInlinesF . mconcat <$> many inline
-     return $ if hasSpace
-                 then (B.space <>) <$> ils
-                 else ils
+  rawAffix False >>= parseFromString (mconcat <$> many inline)
 
 citeStyle :: PandocMonad m => OrgParser m (CiteStyle, [CiteVariant])
-citeStyle = option (DefStyle, []) $ do
-  sty <- option DefStyle $ try $ char '/' *> orgCiteStyle
+citeStyle = do
+  sty <- option NilStyle $ try $ char '/' *> orgCiteStyle
   variants <- option [] $ try $ char '/' *> orgCiteVariants
   return (sty, variants)
 
 orgCiteStyle :: PandocMonad m => OrgParser m CiteStyle
-orgCiteStyle = choice $ map try
-  [ NoAuthorStyle <$ string "noauthor"
-  , NoAuthorStyle <$ string "na"
-  , LocatorsStyle <$ string "locators"
-  , LocatorsStyle <$ char 'l'
-  , NociteStyle <$ string "nocite"
-  , NociteStyle <$ char 'n'
-  , TextStyle <$ string "text"
-  , TextStyle <$ char 't'
-  ]
+orgCiteStyle = try $ do
+  s <- many1 letter
+  case s of
+    "author" -> pure AuthorStyle
+    "a" -> pure AuthorStyle
+    "noauthor" -> pure NoAuthorStyle
+    "na" -> pure NoAuthorStyle
+    "nocite" -> pure NociteStyle
+    "n" -> pure NociteStyle
+    "text" -> pure TextStyle
+    "t" -> pure TextStyle
+    "note" -> pure NoteStyle
+    "ft" -> pure NoteStyle
+    "numeric" -> pure NumericStyle
+    "nb" -> pure NumericStyle
+    "nil" -> pure NilStyle
+    _ -> fail $ "Unknown org cite style " <> show s
 
 orgCiteVariants :: PandocMonad m => OrgParser m [CiteVariant]
 orgCiteVariants =
-  (fullnameVariant `sepBy1` (char '-')) <|> (many1 onecharVariant)
+  (sepBy1' fullnameVariant (char '-')) <|> (many1 onecharVariant)
  where
   fullnameVariant = choice $ map try
      [ Bare <$ string "bare"
@@ -290,11 +290,14 @@ orgCiteVariants =
      ]
 
 data CiteStyle =
-    NoAuthorStyle
+    AuthorStyle
+  | NoAuthorStyle
   | LocatorsStyle
   | NociteStyle
   | TextStyle
-  | DefStyle
+  | NoteStyle
+  | NumericStyle
+  | NilStyle
   deriving Show
 
 data CiteVariant =
@@ -380,8 +383,9 @@ orgRefCiteKey =
       endOfCitation = try $ do
         many $ satisfy isCiteKeySpecialChar
         satisfy $ not . isCiteKeyChar
-  in try $ satisfy isCiteKeyChar `many1TillChar` lookAhead endOfCitation
-
+  in try $ do
+        optional (char '&') -- this is used in org-ref v3
+        satisfy isCiteKeyChar `many1TillChar` lookAhead endOfCitation
 
 -- | Supported citation types.  Only a small subset of org-ref types is
 -- supported for now.  TODO: rewrite this, use LaTeX reader as template.
@@ -508,15 +512,9 @@ internalLink link title = do
 -- @anchor-id@ contains spaces, we are more restrictive in what is accepted as
 -- an anchor.
 anchor :: PandocMonad m => OrgParser m (F Inlines)
-anchor =  try $ do
-  anchorId <- parseAnchor
-  recordAnchorId anchorId
+anchor =  do
+  anchorId <- orgAnchor
   returnF $ B.spanWith (solidify anchorId, [], []) mempty
- where
-       parseAnchor = string "<<"
-                     *> many1Char (noneOf "\t\n\r<>\"' ")
-                     <* string ">>"
-                     <* skipSpaces
 
 -- | Replace every char but [a-zA-Z0-9_.-:] with a hyphen '-'.  This mirrors
 -- the org function @org-export-solidify-link-text@.
@@ -596,11 +594,25 @@ verbatim  = return . B.codeWith ("", ["verbatim"], []) <$> verbatimBetween '='
 code      :: PandocMonad m => OrgParser m (F Inlines)
 code      = return . B.code     <$> verbatimBetween '~'
 
-subscript   :: PandocMonad m => OrgParser m (F Inlines)
-subscript   = fmap B.subscript   <$> try (char '_' *> subOrSuperExpr)
+-- | Returns 'True' if the parser position right after a string, and 'False'
+-- otherwise.
+isAfterString :: PandocMonad m => OrgParser m Bool
+isAfterString = do
+  pos <- getPosition
+  st  <- getState
+  pure $ getLastStrPos st == Just pos
 
+-- | Parses subscript markup. Subscripts must be preceded by a string.
+subscript   :: PandocMonad m => OrgParser m (F Inlines)
+subscript   = do
+  guard =<< isAfterString
+  fmap B.subscript   <$> try (char '_' *> subOrSuperExpr)
+
+-- | Parses superscript markup. Superscript must be preceded by a string.
 superscript :: PandocMonad m => OrgParser m (F Inlines)
-superscript = fmap B.superscript <$> try (char '^' *> subOrSuperExpr)
+superscript = do
+  guard =<< isAfterString
+  fmap B.superscript <$> try (char '^' *> subOrSuperExpr)
 
 math      :: PandocMonad m => OrgParser m (F Inlines)
 math      = return . B.math      <$> choice [ math1CharBetween '$'
@@ -640,11 +652,11 @@ emphasisBetween c = try $ do
 verbatimBetween :: PandocMonad m
                 => Char
                 -> OrgParser m Text
-verbatimBetween c = try $
-  emphasisStart c *>
-  many1TillNOrLessNewlines 1 verbatimChar (emphasisEnd c)
+verbatimBetween c = newlinesToSpaces <$>
+  try (emphasisStart c *> many1TillNOrLessNewlines 1 verbatimChar (emphasisEnd c))
  where
    verbatimChar = noneOf "\n\r" >>= updatePositions
+   newlinesToSpaces = T.map (\d -> if d == '\n' then ' ' else d)
 
 -- | Parses a raw string delimited by @c@ using Org's math rules
 mathTextBetween :: PandocMonad m
@@ -679,7 +691,6 @@ rawMathBetween s e = try $ textStr s *> manyTillChar anyChar (try $ textStr e)
 emphasisStart :: PandocMonad m => Char -> OrgParser m Char
 emphasisStart c = try $ do
   guard =<< afterEmphasisPreChar
-  guard =<< notAfterString
   char c
   lookAhead (noneOf emphasisForbiddenBorderChars)
   pushToInlineCharStack c
@@ -754,7 +765,7 @@ many1TillNOrLessNewlines n p end = try $
 
 -- | Chars not allowed at the (inner) border of emphasis
 emphasisForbiddenBorderChars :: [Char]
-emphasisForbiddenBorderChars = "\t\n\r "
+emphasisForbiddenBorderChars = "\t\n\r \x200B"
 
 -- | The maximum number of newlines within
 emphasisAllowedNewlines :: Int
@@ -792,8 +803,8 @@ notAfterForbiddenBorderChar = do
 subOrSuperExpr :: PandocMonad m => OrgParser m (F Inlines)
 subOrSuperExpr = try $
   simpleSubOrSuperText <|>
-  (choice [ charsInBalanced '{' '}' (noneOf "\n\r")
-          , enclosing ('(', ')') <$> charsInBalanced '(' ')' (noneOf "\n\r")
+  (choice [ charsInBalanced '{' '}' (T.singleton <$> noneOf "\n\r")
+          , enclosing ('(', ')') <$> charsInBalanced '(' ')' (T.singleton <$> noneOf "\n\r")
           ] >>= parseFromString (mconcat <$> many inline))
  where enclosing (left, right) s = T.cons left $ T.snoc s right
 
@@ -814,16 +825,20 @@ inlineLaTeX = try $ do
   allowEntities <- getExportSetting exportWithEntities
   ils <- parseAsInlineLaTeX cmd texOpt
   maybe mzero returnF $
-     parseAsMathMLSym allowEntities cmd `mplus`
-     parseAsMath cmd texOpt `mplus`
-     ils
+    if "\\begin{" `T.isPrefixOf` cmd
+       then ils
+       else parseAsMathMLSym allowEntities cmd `mplus`
+            parseAsMath cmd texOpt `mplus`
+            ils
  where
    parseAsInlineLaTeX :: PandocMonad m
                       => Text -> TeXExport -> OrgParser m (Maybe Inlines)
    parseAsInlineLaTeX cs = \case
-     TeXExport -> maybeRight <$> runParserT inlineCommand state "" (toSources cs)
+     TeXExport -> maybeRight <$> runParserT
+                  (B.rawInline "latex" . snd <$> withRaw inlineCommand)
+                  state "" (toSources cs)
      TeXIgnore -> return (Just mempty)
-     TeXVerbatim -> return (Just $ B.str cs)
+     TeXVerbatim -> return (Just $ B.text cs)
 
    parseAsMathMLSym :: Bool -> Text -> Maybe Inlines
    parseAsMathMLSym allowEntities cs = do
@@ -888,32 +903,31 @@ macro = try $ do
       updateState $ \s -> s { orgStateMacroDepth = recursionDepth }
       return res
  where
-  argument = manyChar $ notFollowedBy eoa *> noneOf ","
+  argument = manyChar $ notFollowedBy eoa *> (escapedComma <|> noneOf ",")
+  escapedComma = try $ char '\\' *> oneOf ",\\"
   eoa = string ")}}}"
 
-smart :: PandocMonad m => OrgParser m (F Inlines)
-smart = choice [doubleQuoted, singleQuoted, orgApostrophe, orgDash, orgEllipses]
+smartQuotes :: PandocMonad m => OrgParser m (F Inlines)
+smartQuotes = do
+  guard =<< getExportSetting exportSmartQuotes
+  choice [doubleQuoted, singleQuoted, orgApostrophe]
   where
-    orgDash = do
-      guardOrSmartEnabled =<< getExportSetting exportSpecialStrings
-      pure <$> dash <* updatePositions '-'
-    orgEllipses = do
-      guardOrSmartEnabled =<< getExportSetting exportSpecialStrings
-      pure <$> ellipses <* updatePositions '.'
     orgApostrophe = do
-      guardEnabled Ext_smart
       (char '\'' <|> char '\8217') <* updateLastPreCharPos
                                    <* updateLastForbiddenCharPos
       returnF (B.str "\x2019")
 
-guardOrSmartEnabled :: PandocMonad m => Bool -> OrgParser m ()
-guardOrSmartEnabled b = do
-  smartExtension <- extensionEnabled Ext_smart <$> getOption readerExtensions
-  guard (b || smartExtension)
+specialStrings :: PandocMonad m => OrgParser m (F Inlines)
+specialStrings = do
+  guard =<< getExportSetting exportSpecialStrings
+  choice [orgDash, orgEllipses, shyHyphen]
+  where
+    shyHyphen   = pure <$> (B.str "\173" <$ string "\\-") <* updatePositions '-'
+    orgDash     = pure <$> dash <* updatePositions '-'
+    orgEllipses = pure <$> ellipses <* updatePositions '.'
 
 singleQuoted :: PandocMonad m => OrgParser m (F Inlines)
 singleQuoted = try $ do
-  guardOrSmartEnabled =<< getExportSetting exportSmartQuotes
   singleQuoteStart
   updatePositions '\''
   withQuoteContext InSingleQuote $
@@ -925,7 +939,6 @@ singleQuoted = try $ do
 -- in the same paragraph.
 doubleQuoted :: PandocMonad m => OrgParser m (F Inlines)
 doubleQuoted = try $ do
-  guardOrSmartEnabled =<< getExportSetting exportSmartQuotes
   doubleQuoteStart
   updatePositions '"'
   contents <- mconcat <$> many (try $ notFollowedBy doubleQuoteEnd >> inline)
