@@ -3,10 +3,10 @@
 {-# LANGUAGE OverloadedStrings #-}
 {- |
    Module      : Text.Pandoc.Readers.Org.Meta
-   Copyright   : Copyright (C) 2014-2022 Albert Krewinkel
+   Copyright   : Copyright (C) 2014-2024 Albert Krewinkel
    License     : GNU GPL, version 2 or above
 
-   Maintainer  : Albert Krewinkel <tarleb+pandoc@moltkeplatz.de>
+   Maintainer  : Albert Krewinkel <albert+pandoc@tarleb.com>
 
 Parsers for Org-mode meta declarations.
 -}
@@ -27,7 +27,7 @@ import qualified Text.Pandoc.Builder as B
 import Text.Pandoc.Class.PandocMonad (PandocMonad)
 import Text.Pandoc.Definition
 import Text.Pandoc.Shared (blocksToInlines, safeRead)
-import Text.Pandoc.Network.HTTP (urlEncode)
+import Text.Pandoc.URI (urlEncode)
 
 import Control.Monad (mzero, void)
 import Data.List (intercalate, intersperse)
@@ -217,7 +217,7 @@ setSelectedTags tags st =
 
 setEmphasisPreChar :: Maybe [Char] -> OrgParserState -> OrgParserState
 setEmphasisPreChar csMb st =
-  let preChars = fromMaybe (orgStateEmphasisPostChars defaultOrgParserState) csMb
+  let preChars = fromMaybe (orgStateEmphasisPreChars defaultOrgParserState) csMb
   in st { orgStateEmphasisPreChars = preChars }
 
 setEmphasisPostChar :: Maybe [Char] -> OrgParserState -> OrgParserState
@@ -252,7 +252,11 @@ todoSequence = try $ do
 
  where
    todoKeyword :: Monad m => OrgParser m Text
-   todoKeyword = many1Char nonspaceChar <* skipSpaces
+   todoKeyword = do
+     keyword <- many1Char nonspaceChar
+     let cleanKeyword = T.takeWhile (/= '(') keyword
+     skipSpaces
+     return cleanKeyword
 
    todoKeywords :: Monad m => OrgParser m [Text]
    todoKeywords = try $
