@@ -3,10 +3,10 @@
 {-# LANGUAGE OverloadedStrings     #-}
 {- |
    Module      : Text.Pandoc.Readers.Org.ParserState
-   Copyright   : Copyright (C) 2014-2022 Albert Krewinkel
+   Copyright   : Copyright (C) 2014-2025 Albert Krewinkel
    License     : GNU GPL, version 2 or above
 
-   Maintainer  : Albert Krewinkel <tarleb+pandoc@moltkeplatz.de>
+   Maintainer  : Albert Krewinkel <albert+pandoc@tarleb.com>
 
 Define the Org-mode parser state.
 -}
@@ -49,14 +49,14 @@ import Data.Text (Text)
 import Text.Pandoc.Builder (Blocks)
 import Text.Pandoc.Definition (Meta (..), nullMeta)
 import Text.Pandoc.Logging
-import Text.Pandoc.Options (ReaderOptions (..))
+import Text.Pandoc.Options (ReaderOptions (..), Extension(..), isEnabled)
 import Text.Pandoc.Parsing (Future, HasIdentifierList (..),
                             HasIncludeFiles (..), HasLastStrPosition (..),
                             HasLogMessages (..), HasMacros (..),
                             HasQuoteContext (..), HasReaderOptions (..),
                             ParserContext (..), QuoteContext (..), SourcePos,
                             askF, asksF, returnF, runF, trimInlinesF)
-import Text.Pandoc.Readers.LaTeX.Types (Macro)
+import Text.Pandoc.TeX (Macro)
 
 -- | This is used to delay evaluation until all relevant information has been
 -- parsed and made available in the parser state.
@@ -164,8 +164,8 @@ instance Default OrgParserState where
 defaultOrgParserState :: OrgParserState
 defaultOrgParserState = OrgParserState
   { orgStateAnchorIds = []
-  , orgStateEmphasisPreChars = "-\t ('\"{"
-  , orgStateEmphasisPostChars  = "-\t\n .,:!?;'\")}["
+  , orgStateEmphasisPreChars = "-\t ('\"{\x200B"
+  , orgStateEmphasisPostChars  = "-\t\n .,:!?;'\")}[\x200B"
   , orgStateEmphasisCharStack = []
   , orgStateEmphasisNewlines = Nothing
   , orgStateExportSettings = def
@@ -193,7 +193,15 @@ defaultOrgParserState = OrgParserState
 
 optionsToParserState :: ReaderOptions -> OrgParserState
 optionsToParserState opts =
-  def { orgStateOptions = opts }
+  let exportSettings = defaultExportSettings
+        { exportSmartQuotes = isEnabled Ext_smart opts ||
+                              isEnabled Ext_smart_quotes opts
+        , exportSpecialStrings = isEnabled Ext_smart opts ||
+                                 isEnabled Ext_special_strings opts
+        }
+  in def { orgStateOptions = opts
+         , orgStateExportSettings = exportSettings
+         }
 
 registerTodoSequence :: TodoSequence -> OrgParserState -> OrgParserState
 registerTodoSequence todoSeq st =
