@@ -43,7 +43,6 @@ import Data.Either (lefts, rights)
 import Data.Maybe (isNothing)
 import qualified Data.Map as M
 import Control.Monad.State
--- import Debug.Trace
 
 isOk :: Char -> Bool
 isOk c = isAscii c && isAlphaNum c
@@ -420,17 +419,22 @@ getData mimetype src
                   return res
                else return raw'
       return $ Fetched (mime, result)
-   handler e = case e of
-                 PandocResourceNotFound r -> do
-                   report $ CouldNotFetchResource r ""
-                   return $ CouldNotFetch e
-                 PandocHttpError u er -> do
-                   report $ CouldNotFetchResource u (tshow er)
-                   return $ CouldNotFetch e
-                 _ -> throwError e
-
-
-
+   handler e
+      -- If fetch failed and we have a fragment and/or query,
+      -- try the fetch again without these, since the resource
+      -- may be local (see #1477, #11021)
+     | T.any (\c -> c == '?' || c == '#') src && not (isURI src)
+       = getData mimetype (removeQueryAndFragment src)
+     | otherwise
+       = case e of
+          PandocResourceNotFound r -> do
+            report $ CouldNotFetchResource r ""
+            return $ CouldNotFetch e
+          PandocHttpError u er -> do
+            report $ CouldNotFetchResource u (tshow er)
+            return $ CouldNotFetch e
+          _ -> throwError e
+   removeQueryAndFragment = T.takeWhile (\c -> c /= '#' && c /= '?')
 
 -- | Convert HTML into self-contained HTML, incorporating images,
 -- scripts, and CSS using data: URIs.
