@@ -4,7 +4,7 @@
 {-# LANGUAGE ViewPatterns        #-}
 {- |
    Module      : Text.Pandoc.Writers.ConTeXt
-   Copyright   : Copyright (C) 2007-2023 John MacFarlane
+   Copyright   : Copyright (C) 2007-2025 John MacFarlane
    License     : GNU GPL, version 2 or above
 
    Maintainer  : John MacFarlane <jgm@berkeley.edu>
@@ -20,7 +20,7 @@ import Control.Monad.State.Strict
 import Data.Char (ord, isDigit)
 import Data.List (intersperse)
 import Data.List.NonEmpty (NonEmpty ((:|)))
-import Data.Maybe (isNothing, mapMaybe, catMaybes)
+import Data.Maybe (mapMaybe, catMaybes)
 import Data.Monoid (Any (Any, getAny))
 import Data.Text (Text)
 import qualified Data.Text as T
@@ -104,6 +104,8 @@ pandocToConTeXt options (Pandoc meta blocks) = do
   mblang <- fromBCP47 (getLang options meta)
   st <- get
   let context =   defField "toc" (writerTableOfContents options)
+                $ defField "lof" (writerListOfFigures options)
+                $ defField "lot" (writerListOfTables options)
                 $ defField "placelist"
                    (mconcat . intersperse ("," :: Doc Text) $
                      take (writerTOCDepth options +
@@ -115,6 +117,7 @@ pandocToConTeXt options (Pandoc meta blocks) = do
                         "subsubsubsection","subsubsubsubsection"])
                 $ defField "body" main
                 $ defField "layout" layoutFromMargins
+                $ defField "tagging" (isEnabled Ext_tagging options)
                 $ defField "number-sections" (writerNumberSections options)
                 $ defField "csl-refs" (stHasCslRefs st)
                 $ defField "csl-hanging-indent" (stCslHangingIndent st)
@@ -128,10 +131,10 @@ pandocToConTeXt options (Pandoc meta blocks) = do
                         _                     -> id)
                 $ defField "emphasis-commands"
                     (mconcat $ Map.elems (stEmphasisCommands st))
-                $ (case writerHighlightStyle options of
-                        Just sty | stHighlighting st ->
-                          defField "highlighting-commands" (styleToConTeXt sty)
-                        _ -> id)
+                $ (case writerHighlightMethod options of
+                      Skylighting sty | stHighlighting st ->
+                        defField "highlighting-commands" (styleToConTeXt sty)
+                      _ -> id)
                 $ (case T.toLower $ lookupMetaString "pdfa" meta of
                         "true" -> resetField "pdfa" (T.pack "1b:2005")
                         _                     -> id) metadata
@@ -230,9 +233,9 @@ blockToConTeXt (CodeBlock (_ident, classes, kv) str) = do
             return (literal h)
   -- blankline because \stoptyping can't have anything after it, inc. '}'
   ($$ blankline) . flush <$>
-    if null classes || isNothing (writerHighlightStyle opts)
-    then pure unhighlighted
-    else highlighted
+    case writerHighlightMethod opts of
+      Skylighting _ | not (null classes) -> pure unhighlighted
+      _ -> highlighted
 blockToConTeXt b@(RawBlock f str)
   | f == Format "context" || f == Format "tex" = return $ literal str <> blankline
   | otherwise = empty <$ report (BlockNotRendered b)
@@ -587,7 +590,7 @@ inlineToConTeXt :: PandocMonad m
                 -> WM m (Doc Text)
 inlineToConTeXt (Emph lst)      = highlightInlines "emph"      "\\em" lst
 inlineToConTeXt (Strong lst)    = highlightInlines "strong"    "\\bf" lst
-inlineToConTeXt (SmallCaps lst) = highlightInlines "smallcaps" "\\sc" lst
+inlineToConTeXt (SmallCaps lst) = highlightInlines "smallcaps" "\\setsmallcaps" lst
 inlineToConTeXt (Underline lst) = do
   contents <- inlineListToConTeXt lst
   return $ "\\underbar" <> braces contents
@@ -622,9 +625,9 @@ inlineToConTeXt (Code (_ident, classes, _kv) str) = do
           Right h -> do
             modify (\st -> st{ stHighlighting = True })
             return (text (T.unpack h))
-  if isNothing (writerHighlightStyle opts) || null classes
-    then rawCode
-    else highlightCode
+  case writerHighlightMethod opts of
+    Skylighting _ | not (null classes) -> highlightCode
+    _ -> rawCode
 inlineToConTeXt (Quoted SingleQuote lst) = do
   contents <- inlineListToConTeXt lst
   return $ "\\quote" <> braces contents
@@ -696,9 +699,9 @@ inlineToConTeXt (Image attr@(_,cls,_) _ (src, _)) = do
       dims = if null dimList
                 then empty
                 else brackets $ mconcat (intersperse "," dimList)
-      clas = if null cls
-                then empty
-                else brackets $ literal $ toLabel $ head cls
+      clas = case cls of
+               [] -> empty
+               (cl:_) -> brackets $ literal $ toLabel cl
       -- Use / for path separators on Windows; see #4918
       fixPathSeparators = T.map $ \c -> case c of
                                           '\\' -> '/'
@@ -829,7 +832,6 @@ fromBCP47' (Just (Lang "grc"_ _ _ _ _))           = Just "agr"
 fromBCP47' (Just (Lang "el" _ _ _ _ _))           = Just "gr"
 fromBCP47' (Just (Lang "eu" _ _ _ _ _))           = Just "ba"
 fromBCP47' (Just (Lang "he" _ _ _ _ _))           = Just "il"
-fromBCP47' (Just (Lang "jp" _ _ _ _ _))           = Just "ja"
 fromBCP47' (Just (Lang "uk" _ _ _ _ _))           = Just "ua"
 fromBCP47' (Just (Lang "vi" _ _ _ _ _))           = Just "vn"
 fromBCP47' (Just (Lang "zh" _ _ _ _ _))           = Just "cn"
