@@ -7,7 +7,7 @@
 {-# LANGUAGE ViewPatterns        #-}
 {- |
    Module      : Text.Pandoc.Writers.LaTeX
-   Copyright   : Copyright (C) 2006-2024 John MacFarlane
+   Copyright   : Copyright (C) 2006-2025 John MacFarlane
    License     : GNU GPL, version 2 or above
 
    Maintainer  : John MacFarlane <jgm@berkeley.edu>
@@ -30,8 +30,9 @@ import Control.Monad
       liftM,
       when,
       unless )
+import Crypto.Hash (hashWith, MD5(MD5))
 import Data.Containers.ListUtils (nubOrd)
-import Data.Char (isDigit)
+import Data.Char (isDigit, isAscii)
 import Data.List (intersperse, (\\))
 import Data.Maybe (catMaybes, fromMaybe, isJust, mapMaybe, isNothing)
 import Data.Monoid (Any (..))
@@ -40,10 +41,11 @@ import qualified Data.Text as T
 import Network.URI (unEscapeString)
 import Text.DocTemplates (FromContext(lookupContext), Val(..), renderTemplate)
 import Text.Collate.Lang (renderLang)
-import Text.Pandoc.Class.PandocMonad (PandocMonad, report, toLang)
+import Text.Pandoc.Class.PandocMonad (PandocMonad, getPOSIXTime, lookupEnv,
+                                      report, toLang)
 import Text.Pandoc.Definition
 import Text.Pandoc.Highlighting (formatLaTeXBlock, formatLaTeXInline, highlight,
-                                 styleToLaTeX)
+                                 defaultStyle, styleToLaTeX)
 import Text.Pandoc.ImageSize
 import Text.Pandoc.Logging
 import Text.Pandoc.Options
@@ -63,7 +65,11 @@ import Text.Pandoc.Writers.LaTeX.Util (stringToLaTeX, StringContext(..),
                                        wrapDiv, hypertarget, labelFor,
                                        getListingsLanguage, mbBraced)
 import Text.Pandoc.Writers.Shared
+import qualified Data.Attoparsec.Text as A
+import qualified Text.Pandoc.UTF8 as UTF8
 import qualified Text.Pandoc.Writers.AnnotatedTable as Ann
+import Data.Char (isLetter)
+import Control.Applicative ((<|>))
 
 -- Work around problems with notes inside emphasis (see #8982)
 isolateBigNotes :: ([Inline] -> Inline) -> [Inline] -> [Inline]
@@ -176,8 +182,22 @@ pandocToLaTeX options (Pandoc meta blocks) = do
   main <- blockListToLaTeX blocks'''
   biblioTitle <- inlineListToLaTeX lastHeader
   st <- get
-  titleMeta <- stringToLaTeX TextString $ stringify $ docTitle meta
+  titleMeta <- escapeCommas <$> -- see #10501
+                stringToLaTeX TextString (stringify $ docTitle meta)
   authorsMeta <- mapM (stringToLaTeX TextString . stringify) $ docAuthors meta
+  -- The trailer ID is as hash used to identify the PDF. Taking control of its
+  -- value is important when aiming for reproducible PDF generation. Setting
+  -- `SOURCE_DATE_EPOCH` is the traditional method used to control
+  -- reproducible builds. There are no cryptographic requirements for the ID,
+  -- so the 128bits (16 bytes) of MD5 are appropriate.
+  reproduciblePDF <- isJust <$> lookupEnv "SOURCE_DATE_EPOCH"
+  trailerID <- do
+    time <- getPOSIXTime
+    let hash = T.pack . show . hashWith MD5 $ mconcat
+               [ UTF8.fromString $ show time
+               , UTF8.fromText $ render Nothing main
+               ]
+    pure $ mconcat [ "<", hash, "> <", hash, ">" ]
   -- we need a default here since lang is used in template conditionals
   let hasStringValue x = isJust (getField x metadata :: Maybe (Doc Text))
   let geometryFromMargins = mconcat $ intersperse ("," :: Doc Text) $
@@ -212,6 +232,7 @@ pandocToLaTeX options (Pandoc meta blocks) = do
                   defField "verbatim-in-note" (stVerbInNote st) $
                   defField "tables" (stTable st) $
                   defField "multirow" (stMultiRow st) $
+                  defField "cancel" (stCancel st) $
                   defField "strikeout" (stStrikeout st) $
                   defField "url" (stUrl st) $
                   defField "numbersections" (writerNumberSections options) $
@@ -221,15 +242,20 @@ pandocToLaTeX options (Pandoc meta blocks) = do
                   defField "svg" (stSVG st) $
                   defField "has-chapters" (stHasChapters st) $
                   defField "has-frontmatter" (documentClass `elem` frontmatterClasses) $
-                  defField "listings" (writerListings options || stLHS st) $
+                  defField "listings" (writerHighlightMethod options ==
+                                       IdiomaticHighlighting
+                                       || stLHS st) $
                   defField "zero-width-non-joiner" (stZwnj st) $
                   defField "beamer" beamer $
                   (if stHighlighting st
-                      then case writerHighlightStyle options of
-                                Just sty ->
+                      then case writerHighlightMethod options of
+                             Skylighting sty ->
                                    defField "highlighting-macros"
                                       (T.stripEnd $ styleToLaTeX sty)
-                                Nothing -> id
+                             DefaultHighlighting ->
+                                   defField "highlighting-macros"
+                                      (T.stripEnd $ styleToLaTeX defaultStyle)
+                             _ -> id
                       else id) $
                   (case writerCiteMethod options of
                          Natbib   -> defField "biblio-title" biblioTitle .
@@ -254,7 +280,10 @@ pandocToLaTeX options (Pandoc meta blocks) = do
                       Just (Just ('A', ds))
                         | not (T.null ds) && T.all isDigit ds
                           -> resetField "papersize" ("a" <> ds)
-                      _   -> id)
+                      _   -> id) .
+                  (if reproduciblePDF
+                    then defField "pdf-trailer-id" trailerID
+                    else id) $
                   metadata
   let babelLang = mblang >>= toBabel
   let context' =
@@ -282,6 +311,10 @@ pandocToLaTeX options (Pandoc meta blocks) = do
        Nothing  -> main
        Just tpl -> renderTemplate tpl context'
 
+-- Commas in title-meta need to be put in braces; see #10501
+escapeCommas :: Text -> Text
+escapeCommas = T.replace "," "{,}"
+
 toSlides :: PandocMonad m => [Block] -> LW m [Block]
 toSlides bs = do
   opts <- gets stOptions
@@ -469,7 +502,8 @@ blockToLaTeX (CodeBlock (identifier,classes,keyvalAttr) str) = do
         ref <- toLabel identifier
         kvs <- mapM (\(k,v) -> (k,) <$>
                        stringToLaTeX TextString v) keyvalAttr
-        let params = if writerListings (stOptions st)
+        let params = if writerHighlightMethod (stOptions st)
+                        == IdiomaticHighlighting
                      then (case getListingsLanguage classes of
                                 Just l  -> [ "language=" <> mbBraced l ]
                                 Nothing -> []) ++
@@ -506,8 +540,11 @@ blockToLaTeX (CodeBlock (identifier,classes,keyvalAttr) str) = do
   case () of
      _ | isEnabled Ext_literate_haskell opts && "haskell" `elem` classes &&
          "literate" `elem` classes           -> lhsCodeBlock
-       | writerListings opts                 -> listingsCodeBlock
-       | not (null classes) && isJust (writerHighlightStyle opts)
+       | writerHighlightMethod opts == IdiomaticHighlighting
+                                             -> listingsCodeBlock
+       | not (null classes), Skylighting _ <- writerHighlightMethod opts
+                                             -> highlightedCodeBlock
+       | not (null classes), DefaultHighlighting <- writerHighlightMethod opts
                                              -> highlightedCodeBlock
        -- we don't want to use \begin{verbatim} if our code
        -- contains \end{verbatim}:
@@ -616,11 +653,14 @@ blockToLaTeX (Header level (id',classes,_) lst) = do
 blockToLaTeX (Table attr blkCapt specs thead tbodies tfoot) =
   tableToLaTeX inlineListToLaTeX blockListToLaTeX
                (Ann.toTable attr blkCapt specs thead tbodies tfoot)
-blockToLaTeX (Figure (ident, _, _) captnode body) = do
+blockToLaTeX (Figure (ident, _, kvs) captnode body) = do
   opts <- gets stOptions
   (capt, captForLof, footnotes) <- getCaption inlineListToLaTeX True captnode
   lab <- labelFor ident
   let caption = "\\caption" <> captForLof <> braces capt <> lab
+      placement = case lookup "latex-placement" kvs of
+        Just p -> brackets (text (T.unpack p))
+        _      -> text ""
 
   isSubfigure <- gets stInFigure
   modify $ \st -> st{ stInFigure = True }
@@ -652,7 +692,7 @@ blockToLaTeX (Figure (ident, _, _) captnode body) = do
           cr <> "\\begin{center}" $$ contents $+$ capt $$ "\\end{center}"
     _ | isSubfigure ->
           innards
-    _ ->  cr <> "\\begin{figure}" $$ innards $$ "\\end{figure}")
+    _ ->  cr <> "\\begin{figure}" <> placement $$ innards $$ "\\end{figure}")
     $$ footnotes
 
 toSubfigure :: PandocMonad m => Int -> Block -> LW m (Doc Text)
@@ -960,12 +1000,14 @@ inlineToLaTeX (Code (_,classes,kvs) str) = do
   -- incorrect results if there is a space (see #5529).
   let inMbox x = "\\mbox" <> braces x
   (if inSoul then inMbox else id) <$>
-   case () of
+   case writerHighlightMethod opts of
      _ | inHeading || inItem  -> rawCode  -- see #5574
-       | writerListings opts  -> listingsCode
-       | isJust (writerHighlightStyle opts) && not (null classes)
+     IdiomaticHighlighting    -> listingsCode
+     Skylighting _ | not (null classes)
                               -> highlightCode
-       | otherwise            -> rawCode
+     DefaultHighlighting | not (null classes)
+                              -> highlightCode
+     _noHighlighting          -> rawCode
 inlineToLaTeX (Quoted qt lst) = do
   contents <- inlineListToLaTeX lst
   csquotes <- liftM stCsquotes get
@@ -1002,10 +1044,16 @@ inlineToLaTeX (Quoted qt lst) = do
 inlineToLaTeX (Str str) = do
   setEmptyLine False
   liftM literal $ stringToLaTeX TextString str
+inlineToLaTeX (Math _ str)
+  | isMathEnv str -- see #9711
+  = do setEmptyLine False
+       when (needsCancel str) $ modify $ \st -> st{ stCancel = True }
+       pure $ literal str
 inlineToLaTeX (Math InlineMath str) = do
   setEmptyLine False
   inSoul <- gets stInSoulCommand
   let contents = literal (handleMathComment str)
+  when (needsCancel str) $ modify $ \st -> st{ stCancel = True }
   return $
     if inSoul -- #9597
        then "$" <> contents <> "$"
@@ -1014,6 +1062,7 @@ inlineToLaTeX (Math DisplayMath str) = do
   setEmptyLine False
   inSoul <- gets stInSoulCommand
   let contents = literal (handleMathComment str)
+  when (needsCancel str) $ modify $ \st -> st{ stCancel = True }
   return $
     if inSoul -- # 9597
        then "$$" <> contents <> "$$"
@@ -1053,7 +1102,9 @@ inlineToLaTeX (Link (id',_,_) txt (src,_)) =
                      then "\\hyperlink" <> braces (literal lab) <> braces contents
                      else "\\hyperref" <> brackets (literal lab) <> braces contents
      _ -> case txt of
-          [Str x] | unEscapeString (T.unpack x) == unEscapeString (T.unpack src) ->  -- autolink
+          [Str x] | T.all isAscii x  -- see #8802
+                  , unEscapeString (T.unpack x) ==
+                    unEscapeString (T.unpack src) ->  -- autolink
                do modify $ \s -> s{ stUrl = True }
                   src' <- stringToLaTeX URLString (escapeURI src)
                   return $ literal $ "\\url{" <> src' <> "}"
@@ -1289,3 +1340,40 @@ ldfLanguages =
   , "galician"
   , "slovene"
   ]
+
+isMathEnv :: Text -> Bool
+isMathEnv t =
+  case T.stripPrefix "\\begin{" t of
+    Nothing -> False
+    Just t' -> T.takeWhile (/= '}') t' `elem`
+      [ "align", "align*"
+      , "flalign", "flalign*"
+      , "alignat", "alignat*"
+      , "dmath", "dmath*"
+      , "dgroup", "dgroup*"
+      , "darray", "darray*"
+      , "gather", "gather*"
+      , "multline", "multline*"
+      , "split"
+      , "subequations"
+      , "equation", "equation*"
+      , "eqnarray"
+      , "displaymath"
+      ]
+
+-- True if the math needs the cancel package
+needsCancel :: Text -> Bool
+needsCancel t =
+  case A.parseOnly pCancel t of
+    Right True -> True
+    _ -> False
+ where
+  pCancel = (False <$ A.endOfInput) <|> do
+    c <- A.anyChar
+    case c of
+      '\\' -> do
+        x <- A.takeWhile isLetter
+        if x == "cancel" || x == "xcancel" || x == "bcancel"
+           then return True
+           else pCancel
+      _ -> pCancel
