@@ -3,11 +3,11 @@
 {- |
    Module      : Text.Pandoc.Writers.Org
    Copyright   : © 2010-2015 Puneeth Chaganti <punchagan@gmail.com>
-                   2010-2023 John MacFarlane <jgm@berkeley.edu>
-                   2016-2023 Albert Krewinkel <tarleb+pandoc@moltkeplatz.de>
+                   2010-2025 John MacFarlane <jgm@berkeley.edu>
+                   2016-2025 Albert Krewinkel <albert+pandoc@tarleb.com>
    License     : GNU GPL, version 2 or above
 
-   Maintainer  : Albert Krewinkel <tarleb+pandoc@moltkeplatz.de>
+   Maintainer  : Albert Krewinkel <albert+pandoc@tarleb.com>
    Stability   : alpha
    Portability : portable
 
@@ -71,6 +71,17 @@ pandocToOrg (Pandoc meta blocks) = do
   let main = body $+$ notes
   let context = defField "body" main
               . defField "math" hasMath
+              . defField "options"
+                         (M.fromList
+                          ((if isEnabled Ext_smart_quotes opts
+                            then (("'", "t"):)
+                            else id) .
+                           (if not (isEnabled Ext_special_strings opts)
+                            then (("-", "nil"):)
+                            else id)
+                           $ ([] :: [(Text, Text)])))
+              . defField "option-special-strings"
+                         (isEnabled Ext_special_strings opts)
               $ metadata
   return $ render colwidth $
     case writerTemplate opts of
@@ -89,20 +100,28 @@ noteToOrg num note = do
   let marker = "[fn:" ++ show num ++ "] "
   return $ hang (length marker) (text marker) contents
 
+-- | Replace Unicode characters with their ASCII representation
+replaceSpecialStrings :: Text -> Text
+replaceSpecialStrings =
+  let expand c = case c of
+        '\x00ad' -> "\\-"
+        '\x2013' -> "--"
+        '\x2014' -> "---"
+        '\x2019' -> "'"
+        '\x2026' -> "..."
+        _        -> T.singleton c
+  in T.concatMap expand
+
 -- | Escape special characters for Org.
 escapeString :: Text -> Doc Text
 escapeString t
   | T.all isAlphaNum t = literal t
   | otherwise = mconcat $ map escChar (T.unpack t)
   where
-   escChar '\x2013' = "--"
-   escChar '\x2014' = "---"
-   escChar '\x2019' = "'"
-   escChar '\x2026' = "..."
-   escChar c
-     -- escape special chars with ZERO WIDTH SPACE as org manual suggests
-     | c == '*' || c == '#' || c == '|' = afterBreak "\x200B" <> char c
-     | otherwise = char c
+    -- escape special chars with ZERO WIDTH SPACE as org manual suggests
+   escChar c = if c == '*' || c == '#' || c == '|'
+     then afterBreak "\x200B" <> char c
+     else char c
 
 isRawFormat :: Format -> Bool
 isRawFormat f =
@@ -488,10 +507,18 @@ inlineToOrg (Subscript lst) = do
 inlineToOrg (SmallCaps lst) = inlineListToOrg lst
 inlineToOrg (Quoted SingleQuote lst) = do
   contents <- inlineListToOrg lst
-  return $ "'" <> contents <> "'"
+  opts <- gets stOptions
+  return $
+    if isEnabled Ext_smart opts || isEnabled Ext_smart_quotes opts
+    then "'" <> contents <> "'"
+    else "‘" <> contents <> "’"
 inlineToOrg (Quoted DoubleQuote lst) = do
   contents <- inlineListToOrg lst
-  return $ "\"" <> contents <> "\""
+  opts <- gets stOptions
+  return $
+    if isEnabled Ext_smart opts || isEnabled Ext_smart_quotes opts
+    then "\"" <> contents <> "\""
+    else "“" <> contents <> "”"
 inlineToOrg (Cite cs lst) = do
   opts <- gets stOptions
   if isEnabled Ext_citations opts
@@ -522,7 +549,12 @@ inlineToOrg (Cite cs lst) = do
        return $ "[cite" <> sty <> ":" <> citeItems <> "]"
      else inlineListToOrg lst
 inlineToOrg (Code _ str) = return $ "=" <> literal str <> "="
-inlineToOrg (Str str) = return $ escapeString str
+inlineToOrg (Str str) = do
+  opts <- gets stOptions
+  let str' = if isEnabled Ext_smart opts || isEnabled Ext_special_strings opts
+             then replaceSpecialStrings str
+             else str
+  return $ escapeString str'
 inlineToOrg (Math t str) = do
   modify $ \st -> st{ stHasMath = True }
   return $ if t == InlineMath
@@ -547,7 +579,7 @@ inlineToOrg (Link _ txt (src, _)) =
   case txt of
         [Str x] | escapeURI x == src ->  -- autolink
              return $ "[[" <> literal (orgPath x) <> "]]"
-        _ -> do contents <- inlineListToOrg txt
+        _ -> do contents <- nowrap <$> inlineListToOrg txt
                 return $ "[[" <> literal (orgPath src) <> "][" <> contents <> "]]"
 inlineToOrg (Image _ _ (source, _)) =
   return $ "[[" <> literal (orgPath source) <> "]]"
