@@ -43,7 +43,7 @@ module Text.Pandoc.Writers.Powerpoint.Presentation ( documentToPresentation
                                                    , LinkTarget(..)
                                                    ) where
 
-
+import Control.Monad
 import Control.Monad.Reader
 import Control.Monad.State
 import Data.List (intercalate)
@@ -244,7 +244,7 @@ data TableProps = TableProps { tblPrFirstRow :: Bool
                              , tblPrBandRow :: Bool
                              } deriving (Show, Eq)
 
-data Graphic = Tbl TableProps [TableCell] [[TableCell]]
+data Graphic = Tbl [Double] TableProps [TableCell] [[TableCell]]
   deriving (Show, Eq)
 
 
@@ -460,15 +460,17 @@ blockToParagraphs (CodeBlock attr str) = do
                                     , pPropIndent = Just 0
                                     }
                 , envRunProps = (envRunProps r){rPropCode = True}}) $ do
-    mbSty <- writerHighlightStyle <$> asks envOpts
+    highlightOpt <- writerHighlightMethod <$> asks envOpts
     synMap <- writerSyntaxMap <$> asks envOpts
-    case mbSty of
-      Just sty ->
-        case highlight synMap (formatSourceLines sty) attr str of
-          Right pElems -> do pPropsNew <- asks envParaProps
-                             return [Paragraph pPropsNew pElems]
-          Left _ -> blockToParagraphs $ Para [Str str]
-      Nothing -> blockToParagraphs $ Para [Str str]
+    let highlightWithStyle style = do
+          case highlight synMap (formatSourceLines style) attr str of
+            Right pElems -> do pPropsNew <- asks envParaProps
+                               return [Paragraph pPropsNew pElems]
+            Left _ -> blockToParagraphs $ Para [Str str]
+    case highlightOpt of
+      Skylighting sty -> highlightWithStyle sty
+      DefaultHighlighting -> highlightWithStyle defaultStyle
+      _ -> blockToParagraphs $ Para [Str str]
 -- We can't yet do incremental lists, but we should render a
 -- (BlockQuote List) as a list to maintain compatibility with other
 -- formats.
@@ -537,7 +539,14 @@ blockToParagraphs (Div (_, classes, _) blks) = let
                    | otherwise -> Nothing
   addIncremental env = env { envInIncrementalDiv = incremental }
   in local addIncremental (concatMapM blockToParagraphs blks)
-blockToParagraphs blk = do
+blockToParagraphs (Figure attr capt blks) = -- This never seems to be used:
+  blockToParagraphs (Shared.figureDiv attr capt blks)
+blockToParagraphs hr@HorizontalRule = notRendered hr
+blockToParagraphs tbl@Table{} = notRendered tbl
+
+-- | Report that a block cannot be rendered.
+notRendered :: Block -> Pres [Paragraph]
+notRendered blk = do
   addLogMessage $ BlockNotRendered blk
   return []
 
@@ -592,8 +601,9 @@ blockToShape (Para (il:_))  | Link _ (il':_) target <- il
       withAttr attr .
       Pic def{picPropLink = Just $ ExternalTarget target} (T.unpack url) title
       <$> inlinesToParElems ils
+blockToShape (Figure _figattr _caption [b]) = blockToShape b
 blockToShape (Table _ blkCapt specs thead tbody tfoot) = do
-  let (caption, algn, _, hdrCells, rows) = toLegacyTable blkCapt specs thead tbody tfoot
+  let (caption, algn, widths, hdrCells, rows) = toLegacyTable blkCapt specs thead tbody tfoot
   caption' <- inlinesToParElems caption
   hdrCells' <- rowToParagraphs algn hdrCells
   rows' <- mapM (rowToParagraphs algn) rows
@@ -605,7 +615,7 @@ blockToShape (Table _ blkCapt specs thead tbody tfoot) = do
                               , tblPrBandRow = True
                               }
 
-  return $ GraphicFrame [Tbl tblPr hdrCells' rows'] caption'
+  return $ GraphicFrame [Tbl widths tblPr hdrCells' rows'] caption'
 -- If the format isn't openxml, we fall through to blockToPargraphs
 blockToShape (RawBlock (Format "openxml") str) = return $ RawOOXMLShape str
 blockToShape blk = do paras <- blockToParagraphs blk
@@ -641,6 +651,7 @@ plainOrPara _ = Nothing
 notText :: Block -> Bool
 notText block | startsWithImage block = True
 notText Table{} = True
+notText Figure{} = True
 notText _ = False
 
 startsWithImage :: Block -> Bool
@@ -1041,9 +1052,9 @@ blockIsBlank
       DefinitionList ds -> all (uncurry (&&) . bimap (all inlineIsBlank) (all (all blockIsBlank))) ds
       Header _ _ ils -> all inlineIsBlank ils
       HorizontalRule -> True
+      Figure _ _ bls -> all blockIsBlank bls
       Table{} -> False
       Div _ bls -> all blockIsBlank bls
-      Null -> True
 
 textIsBlank :: T.Text -> Bool
 textIsBlank = T.all isSpace
