@@ -1,5 +1,6 @@
 {-# LANGUAGE OverloadedStrings   #-}
 {-# LANGUAGE BangPatterns        #-}
+{-# LANGUAGE LambdaCase          #-}
 {-# LANGUAGE ScopedTypeVariables #-}
 {- |
    Module      : Text.Pandoc.Writers.Typst
@@ -28,9 +29,12 @@ import Data.Bifunctor (first, second)
 import Network.URI (unEscapeString)
 import qualified Data.Text as T
 import Control.Monad.State ( StateT, evalStateT, gets, modify )
-import Text.Pandoc.Writers.Shared ( metaToContext, defField, resetField,
-                                    lookupMetaString )
+import Text.Pandoc.Writers.Shared ( lookupMetaInlines, lookupMetaString,
+                                    metaToContext, defField, resetField,
+                                    setupTranslations )
 import Text.Pandoc.Shared (isTightList, orderedListMarkers, tshow)
+import Text.Pandoc.Translations (Term(Abstract), translateTerm)
+import Text.Pandoc.Walk (query)
 import Text.Pandoc.Writers.Math (convertMath)
 import qualified Text.TeXMath as TM
 import Text.DocLayout
@@ -38,8 +42,9 @@ import Text.DocTemplates (renderTemplate)
 import Text.Pandoc.Extensions (Extension(..))
 import Text.Collate.Lang (Lang(..), parseLang)
 import Text.Printf (printf)
-import Data.Char (isAlphaNum, isDigit)
+import Data.Char (isDigit)
 import Data.Maybe (fromMaybe)
+import Unicode.Char (isXIDContinue)
 
 -- | Convert Pandoc to Typst.
 writeTypst :: PandocMonad m => WriterOptions -> Pandoc -> m Text
@@ -64,18 +69,27 @@ pandocToTypst options (Pandoc meta blocks) = do
   let colwidth = if writerWrapText options == WrapAuto
                     then Just $ writerColumns options
                     else Nothing
+  setupTranslations meta
   metadata <- metaToContext options
               blocksToTypst
               (fmap chomp . inlinesToTypst)
               meta
   main <- blocksToTypst blocks
+  abstractTitle <- translateTerm Abstract
   let toPosition :: CaptionPosition -> Text
       toPosition CaptionAbove = "top"
       toPosition CaptionBelow = "bottom"
+  let nociteIds = query (\case
+                           Cite cs _ -> map citationId cs
+                           _         -> [])
+                  $ lookupMetaInlines "nocite" meta
+
   let context = defField "body" main
               $ defField "toc" (writerTableOfContents options)
               $ (if isEnabled Ext_citations options
                     then defField "citations" True
+                       . defField "nocite-ids" (filter (/= "*") nociteIds)
+                       . defField "full-bibliography" ("*" `elem` nociteIds)
                     else id)
               $ (case lookupMetaString "lang" meta of
                     "" -> id
@@ -87,6 +101,7 @@ pandocToTypst options (Pandoc meta blocks) = do
                           maybe id (resetField "region") (langRegion l))
               $ defField "csl" (lookupMetaString "citation-style" meta) -- #10661
               $ defField "smart" (isEnabled Ext_smart options)
+              $ defField "abstract-title" abstractTitle
               $ defField "toc-depth" (tshow $ writerTOCDepth options)
               $ defField "figure-caption-position"
                    (toPosition $ writerFigureCaptionPosition options)
@@ -94,9 +109,9 @@ pandocToTypst options (Pandoc meta blocks) = do
                    (toPosition $ writerTableCaptionPosition options)
               $ defField "page-numbering" ("1" :: Text)
               $ (if writerNumberSections options
-                    then defField "numbering" ("1.1.1.1.1" :: Text)
+                    then defField "section-numbering" ("1.1.1.1.1" :: Text)
                     else id)
-              $ metadata
+              metadata
   return $ render colwidth $
     case writerTemplate options of
        Nothing  -> main
@@ -107,15 +122,15 @@ pickTypstAttrs = foldr go ([],[])
   where
     go (k,v) =
       case T.splitOn ":" k of
-        "typst":"text":x:[] -> second ((x,v):)
-        "typst":x:[] -> first ((x,v):)
+        ["typst", "text", x] -> second ((x,v):)
+        ["typst", x] -> first ((x,v):)
         _ -> id
 
 formatTypstProp :: (Text, Text) -> Text
 formatTypstProp (k,v) = k <> ": " <> v
 
 toTypstPropsListSep :: [(Text, Text)] -> Doc Text
-toTypstPropsListSep = hsep . intersperse "," . (map $ literal . formatTypstProp)
+toTypstPropsListSep = hsep . intersperse "," . map (literal . formatTypstProp)
 
 toTypstPropsListTerm :: [(Text, Text)] -> Doc Text
 toTypstPropsListTerm [] = ""
@@ -147,12 +162,9 @@ blocksToTypst blocks = vcat <$> mapM blockToTypst blocks
 blockToTypst :: PandocMonad m => Block -> TW m (Doc Text)
 blockToTypst block =
   case block of
-    Plain inlines -> do
-      opts <- gets stOptions
-      inlinesToTypst (addLineStartEscapes opts inlines)
+    Plain inlines -> inlinesToTypst inlines
     Para inlines -> do
-      opts <- gets stOptions
-      ($$ blankline) <$> inlinesToTypst (addLineStartEscapes opts inlines)
+      ($$ blankline) <$> inlinesToTypst inlines
     Header level (ident,cls,_) inlines -> do
       contents <- inlinesToTypst inlines
       let lab = toLabel FreestandingLabel ident
@@ -342,9 +354,17 @@ blockToTypst block =
     Div (ident,_,kvs) blocks -> do
       let lab = toLabel FreestandingLabel ident
       let (typstAttrs,typstTextAttrs) = pickTypstAttrs kvs
+      -- Handle lang attribute for Div elements
+      let langAttrs = case lookup "lang" kvs of
+                        Nothing -> []
+                        Just lang -> case parseLang lang of
+                                       Left _ -> []
+                                       Right l -> [("lang",
+                                                    tshow (langLanguage l))]
+      let allTypstTextAttrs = typstTextAttrs ++ langAttrs
       contents <- blocksToTypst blocks
       return $ "#block" <> toTypstPropsListParens typstAttrs <> "["
-        $$ toTypstPoundSetText typstTextAttrs <> contents
+        $$ toTypstPoundSetText allTypstTextAttrs <> contents
         $$ ("]" <+> lab)
 
 defListItemToTypst :: PandocMonad m => ([Inline], [[Block]]) -> TW m (Doc Text)
@@ -483,23 +503,17 @@ mkImage opts useBox src attr
 
 textstyle :: PandocMonad m => Doc Text -> [Inline] -> TW m (Doc Text)
 textstyle s inlines = do
-  opts <-  gets stOptions
-  (<> endCode) . (s <>) . brackets
-    <$> inlinesToTypst (addLineStartEscapes opts inlines)
+  (<> endCode) . (s <>) . brackets . fixInitialAfterBreakEscape
+    <$> inlinesToTypst inlines
 
-addLineStartEscapes :: WriterOptions -> [Inline] -> [Inline]
-addLineStartEscapes opts = go True
- where
-   go True (Str t : xs)
-        | isOrderedListMarker t = RawInline "typst" "\\" : Str t : go False xs
-        | Just (c, t') <- T.uncons t
-        , needsEscapeAtLineStart c
-        , T.null t' = RawInline "typst" "\\" : Str t : go False xs
-   go _ (SoftBreak : xs)
-        | writerWrapText opts == WrapPreserve = SoftBreak : go True xs
-   go _ (LineBreak : xs) = LineBreak : go True xs
-   go _ (x : xs) = x : go False xs
-   go _ [] = []
+fixInitialAfterBreakEscape :: Doc Text -> Doc Text
+fixInitialAfterBreakEscape (Concat x y) =
+  Concat (fixInitialAfterBreakEscape x) y
+-- make an initial AfterBreak escape unconditional (it will be rendered
+-- in a block [..] and there won't be an actual break to trigger it, but
+-- typst still needs the escape)
+fixInitialAfterBreakEscape (AfterBreak "\\") = Text 1 "\\"
+fixInitialAfterBreakEscape x = x
 
 isOrderedListMarker :: Text -> Bool
 isOrderedListMarker t = not (T.null ds) && rest == "."
@@ -509,7 +523,7 @@ escapeTypst :: Bool -> EscapeContext -> Text -> Doc Text
 escapeTypst smart context t =
   (case T.uncons t of
     Just (c, _)
-      | needsEscapeAtLineStart c
+      | needsEscapeAtLineStart c || isOrderedListMarker t
         -> afterBreak "\\"
     _ -> mempty) <>
   (literal (T.replace "//" "\\/\\/"
@@ -570,7 +584,7 @@ toLabel labelType ident
    ident' = T.pack $ unEscapeString $ T.unpack ident
 
 isIdentChar :: Char -> Bool
-isIdentChar c = isAlphaNum c || c == '_' || c == '-' || c == '.' || c == ':'
+isIdentChar c = isXIDContinue c || c == '_' || c == '-' || c == '.' || c == ':'
 
 toCite :: PandocMonad m => Citation -> TW m (Doc Text)
 toCite cite = do
@@ -596,7 +610,10 @@ toCite cite = do
                   [] -> pure mempty
                   suff -> (", supplement: " <>) . brackets
                              <$> inlinesToTypst (eatComma suff)
-       pure $ "#cite" <> parens (label <> form <> suppl) <> endCode
+       pure $ (if citationMode cite == SuppressAuthor  -- see #11044
+                  then parens
+                  else id)
+            $ "#cite" <> parens (label <> form <> suppl) <> endCode
 
 doubleQuoted :: Text -> Doc Text
 doubleQuoted = doubleQuotes . literal . escape
