@@ -1,7 +1,7 @@
 {-# LANGUAGE OverloadedStrings #-}
 {- |
    Module      : Tests.Shared
-   Copyright   : © 2006-2022 John MacFarlane
+   Copyright   : © 2006-2024 John MacFarlane
    License     : GNU GPL, version 2 or above
 
    Maintainer  : John MacFarlane <jgm@berkeley@edu>
@@ -51,7 +51,8 @@ makeSectionsIsIdempotent d =
    in d' == makeSections False Nothing d'
 
 givesTOC :: String -> (Blocks, Blocks) -> TestTree
-givesTOC desc (blocks, toc) = test (toTableOfContents def) desc (toList blocks, head . toList $ toc)
+givesTOC desc (blocks, toc) =
+  test (singleton . toTableOfContents def) desc (toList blocks, toc)
 
 linkId :: T.Text -> T.Text -> T.Text -> Inlines -> Inlines
 linkId lId = linkWith (lId,[],[])
@@ -73,11 +74,11 @@ testTOC = [ givesTOC "empty case" $ mempty =?> bulletList []
               bulletList [plain "H1a" <> bulletList [plain "H2"]]
           , givesTOC "only referenced headers" $
               header 1 "H1a" <> headerId "h2" 2 "H2" =?>
-              bulletList [plain "H1a" <> 
+              bulletList [plain "H1a" <>
                           bulletList [plain $ linkId "toc-h2" "#h2" "" "H2"]]
           , givesTOC "section id used as backup" $
               divWith ("sec",["section"],[]) (header 1 "H1") =?>
-              bulletList [plain $ linkId "toc-sec" "#sec" "" "H1"]             
+              bulletList [plain $ linkId "toc-sec" "#sec" "" "H1"]
           ]
 
 testCollapse :: [TestTree]
